# -*- coding: utf-8 -*-
'''
    :codeauthor: Lukas Raska <lukas@raska.me>
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

from salt.exceptions import CommandExecutionError

# Import Salt Libs
import salt.utils.dictdiffer as dictdiffer
from salt.states import elasticsearch


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ElasticsearchTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.states.elasticsearch
    '''
    def setup_loader_modules(self):
        return {
            elasticsearch: {
                '__opts__': {'test': False},
                '__utils__': {'dictdiffer.deep_diff': dictdiffer.deep_diff}
            }
        }

    # 'index_absent' function tests: 1

    def test_index_absent(self):
        '''
        Test to manage a elasticsearch index.
        '''
        name = 'foo'

        ret = {'name': name,
               'result': True,
               'comment': 'Index foo is already absent',
               'changes': {}}

        mock_get = MagicMock(side_effect=[None, {name: {"test": "key"}}, {name: {}}, {name: {"test": "key"}}, CommandExecutionError, {name: {"test": "key"}}])
        mock_delete = MagicMock(side_effect=[True, False, CommandExecutionError])

        with patch.dict(elasticsearch.__salt__, {'elasticsearch.index_get': mock_get,
                                             'elasticsearch.index_delete': mock_delete}):
            self.assertDictEqual(elasticsearch.index_absent(name), ret)

            ret.update({'comment': 'Successfully removed index foo', 'changes': {"old": {"test": "key"}}})
            self.assertDictEqual(elasticsearch.index_absent(name), ret)

            ret.update({'comment': 'Failed to remove index foo for unknown reasons', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.index_absent(name), ret)

            with patch.dict(elasticsearch.__opts__, {'test': True}):
                ret.update({'comment': "Index foo will be removed", 'result': None, 'changes': {"old": {"test": "key"}}})
                self.assertDictEqual(elasticsearch.index_absent(name), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.index_absent(name), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.index_absent(name), ret)

    # 'index_present' function tests: 1

    def test_index_present(self):
        '''
        Test to manage a elasticsearch index.
        '''
        name = 'foo'

        ret = {'name': name,
               'result': True,
               'comment': 'Index foo is already present',
               'changes': {}}

        mock_exists = MagicMock(side_effect=[True, False, False, False, CommandExecutionError, False, False])
        mock_get = MagicMock(side_effect=[{name: {"test": "key"}}, CommandExecutionError])
        mock_create = MagicMock(side_effect=[True, False, CommandExecutionError, True])

        with patch.dict(elasticsearch.__salt__, {'elasticsearch.index_get': mock_get,
                                             'elasticsearch.index_exists': mock_exists,
                                             'elasticsearch.index_create': mock_create}):
            self.assertDictEqual(elasticsearch.index_present(name), ret)

            ret.update({'comment': 'Successfully created index foo', 'changes': {"new": {"test": "key"}}})
            self.assertDictEqual(elasticsearch.index_present(name), ret)

            ret.update({'comment': 'Cannot create index foo, False', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.index_present(name), ret)

            with patch.dict(elasticsearch.__opts__, {'test': True}):
                ret.update({'comment': "Index foo does not exist and will be created", 'result': None, 'changes': {"new": {"test2": "key"}}})
                self.assertDictEqual(elasticsearch.index_present(name, {"test2": "key"}), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.index_absent(name), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.index_absent(name), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.index_absent(name), ret)

    # 'alias_absent' function tests: 1

    def test_alias_absent(self):
        '''
        Test to manage a elasticsearch alias.
        '''
        name = 'foo'
        index = 'bar'

        alias = {index: {"aliases": {name: {"test": "key"}}}}

        ret = {'name': name,
               'result': True,
               'comment': 'Alias foo for index bar is already absent',
               'changes': {}}

        mock_get = MagicMock(side_effect=[None, {"foo2": {}}, alias, alias, alias, CommandExecutionError, alias])
        mock_delete = MagicMock(side_effect=[True, False, CommandExecutionError])

        with patch.dict(elasticsearch.__salt__, {'elasticsearch.alias_get': mock_get,
                                             'elasticsearch.alias_delete': mock_delete}):
            self.assertDictEqual(elasticsearch.alias_absent(name, index), ret)
            self.assertDictEqual(elasticsearch.alias_absent(name, index), ret)

            ret.update({'comment': 'Successfully removed alias foo for index bar', 'changes': {"old": {"test": "key"}}})
            self.assertDictEqual(elasticsearch.alias_absent(name, index), ret)

            ret.update({'comment': 'Failed to remove alias foo for index bar for unknown reasons', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.alias_absent(name, index), ret)

            with patch.dict(elasticsearch.__opts__, {'test': True}):
                ret.update({'comment': "Alias foo for index bar will be removed", 'result': None, 'changes': {"old": {"test": "key"}}})
                self.assertDictEqual(elasticsearch.alias_absent(name, index), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.alias_absent(name, index), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.alias_absent(name, index), ret)

    # 'alias_present' function tests: 1

    def test_alias_present(self):
        '''
        Test to manage a elasticsearch alias.
        '''
        name = 'foo'
        index = 'bar'

        alias = {index: {"aliases": {name: {"test": "key"}}}}

        ret = {'name': name,
               'result': True,
               'comment': 'Alias foo for index bar is already present',
               'changes': {}}

        mock_get = MagicMock(side_effect=[alias, alias, None, None, None, alias, CommandExecutionError, None])
        mock_create = MagicMock(side_effect=[True, True, False, CommandExecutionError])

        with patch.dict(elasticsearch.__salt__, {'elasticsearch.alias_get': mock_get,
                                                 'elasticsearch.alias_create': mock_create}):
            self.assertDictEqual(elasticsearch.alias_present(name, index, {"test": "key"}), ret)

            ret.update({'comment': "Successfully replaced alias foo for index bar", 'changes': {'old': {"test": "key"}, 'new': {"test2": "key"}}})
            self.assertDictEqual(elasticsearch.alias_present(name, index, {"test2": "key"}), ret)

            ret.update({'comment': "Successfully created alias foo for index bar", 'changes': {'new': {"test2": "key"}}})
            self.assertDictEqual(elasticsearch.alias_present(name, index, {"test2": "key"}), ret)

            ret.update({'comment': 'Cannot create alias foo for index bar, False', 'result': False})
            self.assertDictEqual(elasticsearch.alias_present(name, index, {"test2": "key"}), ret)

            with patch.dict(elasticsearch.__opts__, {'test': True}):
                ret.update({'comment': "Alias foo for index bar does not exist and will be created", 'result': None, 'changes': {'new': {"test2": "key"}}})
                self.assertDictEqual(elasticsearch.alias_present(name, index, {"test2": "key"}), ret)

                ret.update({'comment': "Alias foo for index bar exists with wrong configuration and will be overridden", 'result': None, 'changes': {'old': {"test": "key"}, 'new': {"test2": "key"}}})
                self.assertDictEqual(elasticsearch.alias_present(name, index, {"test2": "key"}), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.alias_present(name, index), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.alias_present(name, index), ret)

    # 'index_template_absent' function tests: 1

    def test_index_template_absent(self):
        '''
        Test to manage a elasticsearch index template.
        '''
        name = 'foo'

        index_template = {name: {"test": "key"}}

        ret = {'name': name,
               'result': True,
               'comment': 'Index template foo is already absent',
               'changes': {}}

        mock_get = MagicMock(side_effect=[None, {"bar": {}}, index_template, index_template, index_template, CommandExecutionError, index_template])
        mock_delete = MagicMock(side_effect=[True, False, CommandExecutionError])

        with patch.dict(elasticsearch.__salt__, {'elasticsearch.index_template_get': mock_get,
                                             'elasticsearch.index_template_delete': mock_delete}):
            self.assertDictEqual(elasticsearch.index_template_absent(name), ret)
            self.assertDictEqual(elasticsearch.index_template_absent(name), ret)

            ret.update({'comment': 'Successfully removed index template foo', 'changes': {"old": {"test": "key"}}})
            self.assertDictEqual(elasticsearch.index_template_absent(name), ret)

            ret.update({'comment': 'Failed to remove index template foo for unknown reasons', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.index_template_absent(name), ret)

            with patch.dict(elasticsearch.__opts__, {'test': True}):
                ret.update({'comment': "Index template foo will be removed", 'result': None, 'changes': {"old": {"test": "key"}}})
                self.assertDictEqual(elasticsearch.index_template_absent(name), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.index_template_absent(name), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.index_template_absent(name), ret)

    # 'index_template_present' function tests: 1

    def test_index_template_present(self):
        '''
        Test to manage a elasticsearch index template.
        '''
        name = 'foo'

        index_template = {name: {"test": "key"}}

        ret = {'name': name,
               'result': True,
               'comment': 'Index template foo is already present',
               'changes': {}}

        mock_exists = MagicMock(side_effect=[True, False, False, False, CommandExecutionError, False, False])
        mock_create = MagicMock(side_effect=[True, False, CommandExecutionError, True])
        mock_get = MagicMock(side_effect=[index_template, CommandExecutionError])

        with patch.dict(elasticsearch.__salt__, {'elasticsearch.index_template_get': mock_get,
                                             'elasticsearch.index_template_create': mock_create,
                                             'elasticsearch.index_template_exists': mock_exists}):
            self.assertDictEqual(elasticsearch.index_template_present(name, {"test2": "key"}), ret)

            ret.update({'comment': "Successfully created index template foo", 'changes': {'new': {"test": "key"}}})
            self.assertDictEqual(elasticsearch.index_template_present(name, {"test2": "key"}), ret)

            ret.update({'comment': 'Cannot create index template foo, False', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.index_template_present(name, {"test2": "key"}), ret)

            with patch.dict(elasticsearch.__opts__, {'test': True}):
                ret.update({'comment': "Index template foo does not exist and will be created", 'result': None, 'changes': {'new': {"test2": "key"}}})
                self.assertDictEqual(elasticsearch.index_template_present(name, {"test2": "key"}), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.index_template_present(name, {}), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.index_template_present(name, {}), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.index_template_present(name, {}), ret)

    # 'pipeline_absent' function tests: 1

    def test_pipeline_absent(self):
        '''
        Test to manage a elasticsearch pipeline.
        '''
        name = 'foo'

        pipeline = {name: {"test": "key"}}

        ret = {'name': name,
               'result': True,
               'comment': 'Pipeline foo is already absent',
               'changes': {}}

        mock_get = MagicMock(side_effect=[None, {"foo2": {}}, pipeline, pipeline, pipeline, CommandExecutionError, pipeline])
        mock_delete = MagicMock(side_effect=[True, False, CommandExecutionError])

        with patch.dict(elasticsearch.__salt__, {'elasticsearch.pipeline_get': mock_get,
                                             'elasticsearch.pipeline_delete': mock_delete}):
            self.assertDictEqual(elasticsearch.pipeline_absent(name), ret)
            self.assertDictEqual(elasticsearch.pipeline_absent(name), ret)

            ret.update({'comment': 'Successfully removed pipeline foo', 'changes': {"old": {"test": "key"}}})
            self.assertDictEqual(elasticsearch.pipeline_absent(name), ret)

            ret.update({'comment': 'Failed to remove pipeline foo for unknown reasons', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.pipeline_absent(name), ret)

            with patch.dict(elasticsearch.__opts__, {'test': True}):
                ret.update({'comment': "Pipeline foo will be removed", 'result': None, 'changes': {"old": {"test": "key"}}})
                self.assertDictEqual(elasticsearch.pipeline_absent(name), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.pipeline_absent(name), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.pipeline_absent(name), ret)

    # 'pipeline_present' function tests: 1

    def test_pipeline_present(self):
        '''
        Test to manage a elasticsearch pipeline.
        '''
        name = 'foo'

        pipeline = {name: {"test": "key"}}

        ret = {'name': name,
               'result': True,
               'comment': 'Pipeline foo is already present',
               'changes': {}}

        mock_get = MagicMock(side_effect=[pipeline, pipeline, None, None, None, pipeline, CommandExecutionError, None])
        mock_create = MagicMock(side_effect=[True, True, False, CommandExecutionError])

        with patch.dict(elasticsearch.__salt__, {'elasticsearch.pipeline_get': mock_get,
                                             'elasticsearch.pipeline_create': mock_create}):
            self.assertDictEqual(elasticsearch.pipeline_present(name, {"test": "key"}), ret)

            ret.update({'comment': "Successfully replaced pipeline foo", 'changes': {'old': {"test": "key"}, 'new': {"test2": "key"}}})
            self.assertDictEqual(elasticsearch.pipeline_present(name, {"test2": "key"}), ret)

            ret.update({'comment': "Successfully created pipeline foo", 'changes': {'new': {"test2": "key"}}})
            self.assertDictEqual(elasticsearch.pipeline_present(name, {"test2": "key"}), ret)

            ret.update({'comment': 'Cannot create pipeline foo, False', 'result': False})
            self.assertDictEqual(elasticsearch.pipeline_present(name, {"test2": "key"}), ret)

            with patch.dict(elasticsearch.__opts__, {'test': True}):
                ret.update({'comment': "Pipeline foo does not exist and will be created", 'result': None, 'changes': {'new': {"test2": "key"}}})
                self.assertDictEqual(elasticsearch.pipeline_present(name, {"test2": "key"}), ret)

                ret.update({'comment': "Pipeline foo exists with wrong configuration and will be overridden", 'result': None, 'changes': {'old': {"test": "key"}, 'new': {"test2": "key"}}})
                self.assertDictEqual(elasticsearch.pipeline_present(name, {"test2": "key"}), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.pipeline_present(name, {}), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.pipeline_present(name, {}), ret)

    # 'search_template_absent' function tests: 1

    def test_search_template_absent(self):
        '''
        Test to manage a elasticsearch search template.
        '''
        name = 'foo'

        template = {"template": '{"test": "key"}'}

        ret = {'name': name,
               'result': True,
               'comment': 'Search template foo is already absent',
               'changes': {}}

        mock_get = MagicMock(side_effect=[None, template, template, template, CommandExecutionError, template])
        mock_delete = MagicMock(side_effect=[True, False, CommandExecutionError])

        with patch.dict(elasticsearch.__salt__, {'elasticsearch.search_template_get': mock_get,
                                             'elasticsearch.search_template_delete': mock_delete}):
            self.assertDictEqual(elasticsearch.search_template_absent(name), ret)

            ret.update({'comment': 'Successfully removed search template foo', 'changes': {"old": {"test": "key"}}})
            self.assertDictEqual(elasticsearch.search_template_absent(name), ret)

            ret.update({'comment': 'Failed to remove search template foo for unknown reasons', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.search_template_absent(name), ret)

            with patch.dict(elasticsearch.__opts__, {'test': True}):
                ret.update({'comment': "Search template foo will be removed", 'result': None, 'changes': {"old": {"test": "key"}}})
                self.assertDictEqual(elasticsearch.search_template_absent(name), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.search_template_absent(name), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.search_template_absent(name), ret)

    # 'pipeline_present' function tests: 1

    def test_search_template_present(self):
        '''
        Test to manage a elasticsearch search template.
        '''
        name = 'foo'

        template = {"template": '{"test": "key"}'}

        ret = {'name': name,
               'result': True,
               'comment': 'Search template foo is already present',
               'changes': {}}

        mock_get = MagicMock(side_effect=[template, template, None, None, None, template, CommandExecutionError, None])
        mock_create = MagicMock(side_effect=[True, True, False, CommandExecutionError])

        with patch.dict(elasticsearch.__salt__, {'elasticsearch.search_template_get': mock_get,
                                             'elasticsearch.search_template_create': mock_create}):
            self.assertDictEqual(elasticsearch.search_template_present(name, {"test": "key"}), ret)

            ret.update({'comment': "Successfully replaced search template foo", 'changes': {'old': {"test": "key"}, 'new': {"test2": "key"}}})
            self.assertDictEqual(elasticsearch.search_template_present(name, {"test2": "key"}), ret)

            ret.update({'comment': "Successfully created search template foo", 'changes': {'new': {"test2": "key"}}})
            self.assertDictEqual(elasticsearch.search_template_present(name, {"test2": "key"}), ret)

            ret.update({'comment': 'Cannot create search template foo, False', 'result': False})
            self.assertDictEqual(elasticsearch.search_template_present(name, {"test2": "key"}), ret)

            with patch.dict(elasticsearch.__opts__, {'test': True}):
                ret.update({'comment': "Search template foo does not exist and will be created", 'result': None, 'changes': {'new': {"test2": "key"}}})
                self.assertDictEqual(elasticsearch.search_template_present(name, {"test2": "key"}), ret)

                ret.update({'comment': "Search template foo exists with wrong configuration and will be overridden", 'result': None, 'changes': {'old': {"test": "key"}, 'new': {"test2": "key"}}})
                self.assertDictEqual(elasticsearch.search_template_present(name, {"test2": "key"}), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.search_template_present(name, {}), ret)

            ret.update({'comment': '', 'result': False, 'changes': {}})
            self.assertDictEqual(elasticsearch.search_template_present(name, {}), ret)



# -*- coding: utf-8 -*-
#
# Author: Bo Maryniuk <bo@suse.de>
#
# Copyright 2017 SUSE LLC
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Import Salt Testing Libs
from __future__ import absolute_import, print_function, unicode_literals
import os
try:
    import pytest
except ImportError as import_error:
    pytest = None
NO_PYTEST = not bool(pytest)

from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    patch,
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON
)

import salt.modules.ansiblegate as ansible
import salt.utils.platform
from salt.exceptions import LoaderError


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(NO_PYTEST, False)
@skipIf(salt.utils.platform.is_windows(), 'Not supported on Windows')
class AnsiblegateTestCase(TestCase, LoaderModuleMockMixin):
    def setUp(self):
        self.resolver = ansible.AnsibleModuleResolver({})
        self.resolver._modules_map = {
            'one.two.three': os.sep + os.path.join('one', 'two', 'three.py'),
            'four.five.six': os.sep + os.path.join('four', 'five', 'six.py'),
            'three.six.one': os.sep + os.path.join('three', 'six', 'one.py'),
        }

    def tearDown(self):
        self.resolver = None

    def setup_loader_modules(self):
        return {ansible: {}}

    def test_ansible_module_help(self):
        '''
        Test help extraction from the module
        :return:
        '''
        class Module(object):
            '''
            An ansible module mock.
            '''
            __name__ = 'foo'
            DOCUMENTATION = """
---
one:
   text here
---
two:
   text here
description:
   describe the second part
        """

        with patch.object(ansible, '_resolver', self.resolver), \
            patch.object(ansible._resolver, 'load_module', MagicMock(return_value=Module())):
            ret = ansible.help('dummy')
            assert sorted(ret.get('Available sections on module "{0}"'.format(
                Module().__name__))) == ['one', 'two']
            assert ret.get('Description') == 'describe the second part'

    def test_module_resolver_modlist(self):
        '''
        Test Ansible resolver modules list.
        :return:
        '''
        assert self.resolver.get_modules_list() == ['four.five.six', 'one.two.three', 'three.six.one']
        for ptr in ['five', 'fi', 've']:
            assert self.resolver.get_modules_list(ptr) == ['four.five.six']
        for ptr in ['si', 'ix', 'six']:
            assert self.resolver.get_modules_list(ptr) == ['four.five.six', 'three.six.one']
        assert self.resolver.get_modules_list('one') == ['one.two.three', 'three.six.one']
        assert self.resolver.get_modules_list('one.two') == ['one.two.three']
        assert self.resolver.get_modules_list('four') == ['four.five.six']

    def test_resolver_module_loader_failure(self):
        '''
        Test Ansible module loader.
        :return:
        '''
        mod = 'four.five.six'
        with pytest.raises(ImportError) as import_error:
            self.resolver.load_module(mod)

        mod = 'i.even.do.not.exist.at.all'
        with pytest.raises(LoaderError) as loader_error:
            self.resolver.load_module(mod)

    def test_resolver_module_loader(self):
        '''
        Test Ansible module loader.
        :return:
        '''
        with patch('salt.modules.ansiblegate.importlib', MagicMock()),\
            patch('salt.modules.ansiblegate.importlib.import_module', lambda x: x):
            assert self.resolver.load_module('four.five.six') == 'ansible.modules.four.five.six'

    def test_resolver_module_loader_import_failure(self):
        '''
        Test Ansible module loader failure.
        :return:
        '''
        with patch('salt.modules.ansiblegate.importlib', MagicMock()),\
            patch('salt.modules.ansiblegate.importlib.import_module', lambda x: x):
            with pytest.raises(LoaderError) as loader_error:
                self.resolver.load_module('something.strange')

    def test_virtual_function_no_ansible_installed(self):
        '''
        Test Ansible module __virtual__ when ansible is not installed on the minion.
        :return:
        '''
        with patch('salt.modules.ansiblegate.ansible', None):
            assert ansible.__virtual__() == (False, 'Ansible is not installed on this system')

    @patch('salt.modules.ansiblegate.ansible', MagicMock())
    @patch('salt.modules.ansiblegate.list', MagicMock())
    @patch('salt.modules.ansiblegate._set_callables', MagicMock())
    @patch('salt.modules.ansiblegate.AnsibleModuleCaller', MagicMock())
    def test_virtual_function_ansible_is_installed(self):
        '''
        Test Ansible module __virtual__ when ansible is installed on the minion.
        :return:
        '''
        resolver = MagicMock()
        resolver.resolve = MagicMock()
        resolver.resolve.install = MagicMock()
        with patch('salt.modules.ansiblegate.AnsibleModuleResolver', resolver):
            assert ansible.__virtual__() == (True, None)



# -*- coding: utf-8 -*-
#
# Author: Bo Maryniuk <bo@suse.de>
#
# Copyright 2018 SUSE LLC
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Import Salt Testing Libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import tempfile

try:
    import pytest
except ImportError as import_error:
    pytest = None

from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    patch,
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON
)

from salt.modules import x509
import salt.utils.stringutils
import salt.utils.files

try:
    import M2Crypto  # pylint: disable=unused-import
    HAS_M2CRYPTO = True
except ImportError:
    HAS_M2CRYPTO = False


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not bool(pytest), False)
class X509TestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        return {x509: {}}

    @patch('salt.modules.x509.log', MagicMock())
    def test_private_func__parse_subject(self):
        '''
        Test private function _parse_subject(subject) it handles a missing fields
        :return:
        '''
        class FakeSubject(object):
            '''
            Class for faking x509'th subject.
            '''
            def __init__(self):
                self.nid = {'Darth Vader': 1}

            def __getattr__(self, item):
                if item != 'nid':
                    raise TypeError('A star wars satellite accidentally blew up the WAN.')

        subj = FakeSubject()
        x509._parse_subject(subj)
        assert x509.log.trace.call_args[0][0] == "Missing attribute '%s'. Error: %s"
        assert x509.log.trace.call_args[0][1] == list(subj.nid.keys())[0]
        assert isinstance(x509.log.trace.call_args[0][2], TypeError)

    @skipIf(not HAS_M2CRYPTO, 'Skipping, M2Crypto is unavailble')
    def test_get_pem_entry(self):
        '''
        Test private function _parse_subject(subject) it handles a missing fields
        :return:
        '''
        ca_key = b'''-----BEGIN RSA PRIVATE KEY-----
MIICWwIBAAKBgQCjdjbgL4kQ8Lu73xeRRM1q3C3K3ptfCLpyfw38LRnymxaoJ6ls
pNSx2dU1uJ89YKFlYLo1QcEk4rJ2fdIjarV0kuNCY3rC8jYUp9BpAU5Z6p9HKeT1
2rTPH81JyjbQDR5PyfCyzYOQtpwpB4zIUUK/Go7tTm409xGKbbUFugJNgQIDAQAB
AoGAF24we34U1ZrMLifSRv5nu3OIFNZHyx2DLDpOFOGaII5edwgIXwxZeIzS5Ppr
yO568/8jcdLVDqZ4EkgCwRTgoXRq3a1GLHGFmBdDNvWjSTTMLoozuM0t2zjRmIsH
hUd7tnai9Lf1Bp5HlBEhBU2gZWk+SXqLvxXe74/+BDAj7gECQQDRw1OPsrgTvs3R
3MNwX6W8+iBYMTGjn6f/6rvEzUs/k6rwJluV7n8ISNUIAxoPy5g5vEYK6Ln/Ttc7
u0K1KNlRAkEAx34qcxjuswavL3biNGE+8LpDJnJx1jaNWoH+ObuzYCCVMusdT2gy
kKuq9ytTDgXd2qwZpIDNmscvReFy10glMQJAXebMz3U4Bk7SIHJtYy7OKQzn0dMj
35WnRV81c2Jbnzhhu2PQeAvt/i1sgEuzLQL9QEtSJ6wLJ4mJvImV0TdaIQJAAYyk
TcKK0A8kOy0kMp3yvDHmJZ1L7wr7bBGIZPBlQ0Ddh8i1sJExm1gJ+uN2QKyg/XrK
tDFf52zWnCdVGgDwcQJALW/WcbSEK+JVV6KDJYpwCzWpKIKpBI0F6fdCr1G7Xcwj
c9bcgp7D7xD+TxWWNj4CSXEccJgGr91StV+gFg4ARQ==
-----END RSA PRIVATE KEY-----
'''

        ret = x509.get_pem_entry(ca_key)
        self.assertEqual(ret, ca_key)

    @skipIf(not HAS_M2CRYPTO, 'Skipping, M2Crypto is unavailble')
    def test_get_private_key_size(self):
        '''
        Test private function _parse_subject(subject) it handles a missing fields
        :return:
        '''
        ca_key = '''
-----BEGIN RSA PRIVATE KEY-----
MIICWwIBAAKBgQCjdjbgL4kQ8Lu73xeRRM1q3C3K3ptfCLpyfw38LRnymxaoJ6ls
pNSx2dU1uJ89YKFlYLo1QcEk4rJ2fdIjarV0kuNCY3rC8jYUp9BpAU5Z6p9HKeT1
2rTPH81JyjbQDR5PyfCyzYOQtpwpB4zIUUK/Go7tTm409xGKbbUFugJNgQIDAQAB
AoGAF24we34U1ZrMLifSRv5nu3OIFNZHyx2DLDpOFOGaII5edwgIXwxZeIzS5Ppr
yO568/8jcdLVDqZ4EkgCwRTgoXRq3a1GLHGFmBdDNvWjSTTMLoozuM0t2zjRmIsH
hUd7tnai9Lf1Bp5HlBEhBU2gZWk+SXqLvxXe74/+BDAj7gECQQDRw1OPsrgTvs3R
3MNwX6W8+iBYMTGjn6f/6rvEzUs/k6rwJluV7n8ISNUIAxoPy5g5vEYK6Ln/Ttc7
u0K1KNlRAkEAx34qcxjuswavL3biNGE+8LpDJnJx1jaNWoH+ObuzYCCVMusdT2gy
kKuq9ytTDgXd2qwZpIDNmscvReFy10glMQJAXebMz3U4Bk7SIHJtYy7OKQzn0dMj
35WnRV81c2Jbnzhhu2PQeAvt/i1sgEuzLQL9QEtSJ6wLJ4mJvImV0TdaIQJAAYyk
TcKK0A8kOy0kMp3yvDHmJZ1L7wr7bBGIZPBlQ0Ddh8i1sJExm1gJ+uN2QKyg/XrK
tDFf52zWnCdVGgDwcQJALW/WcbSEK+JVV6KDJYpwCzWpKIKpBI0F6fdCr1G7Xcwj
c9bcgp7D7xD+TxWWNj4CSXEccJgGr91StV+gFg4ARQ==
-----END RSA PRIVATE KEY-----
'''

        ret = x509.get_private_key_size(ca_key)
        self.assertEqual(ret, 1024)

    @skipIf(not HAS_M2CRYPTO, 'Skipping, M2Crypto is unavailble')
    def test_create_key(self):
        '''
        Test that x509.create_key returns a private key
        :return:
        '''
        ret = x509.create_private_key(text=True,
                                      passphrase='super_secret_passphrase')
        self.assertIn('BEGIN RSA PRIVATE KEY', ret)

    @skipIf(not HAS_M2CRYPTO, 'Skipping, M2Crypto is unavailble')
    def test_create_certificate(self):
        '''
        Test private function _parse_subject(subject) it handles a missing fields
        :return:
        '''
        ca_key = '''
-----BEGIN RSA PRIVATE KEY-----
MIICWwIBAAKBgQCjdjbgL4kQ8Lu73xeRRM1q3C3K3ptfCLpyfw38LRnymxaoJ6ls
pNSx2dU1uJ89YKFlYLo1QcEk4rJ2fdIjarV0kuNCY3rC8jYUp9BpAU5Z6p9HKeT1
2rTPH81JyjbQDR5PyfCyzYOQtpwpB4zIUUK/Go7tTm409xGKbbUFugJNgQIDAQAB
AoGAF24we34U1ZrMLifSRv5nu3OIFNZHyx2DLDpOFOGaII5edwgIXwxZeIzS5Ppr
yO568/8jcdLVDqZ4EkgCwRTgoXRq3a1GLHGFmBdDNvWjSTTMLoozuM0t2zjRmIsH
hUd7tnai9Lf1Bp5HlBEhBU2gZWk+SXqLvxXe74/+BDAj7gECQQDRw1OPsrgTvs3R
3MNwX6W8+iBYMTGjn6f/6rvEzUs/k6rwJluV7n8ISNUIAxoPy5g5vEYK6Ln/Ttc7
u0K1KNlRAkEAx34qcxjuswavL3biNGE+8LpDJnJx1jaNWoH+ObuzYCCVMusdT2gy
kKuq9ytTDgXd2qwZpIDNmscvReFy10glMQJAXebMz3U4Bk7SIHJtYy7OKQzn0dMj
35WnRV81c2Jbnzhhu2PQeAvt/i1sgEuzLQL9QEtSJ6wLJ4mJvImV0TdaIQJAAYyk
TcKK0A8kOy0kMp3yvDHmJZ1L7wr7bBGIZPBlQ0Ddh8i1sJExm1gJ+uN2QKyg/XrK
tDFf52zWnCdVGgDwcQJALW/WcbSEK+JVV6KDJYpwCzWpKIKpBI0F6fdCr1G7Xcwj
c9bcgp7D7xD+TxWWNj4CSXEccJgGr91StV+gFg4ARQ==
-----END RSA PRIVATE KEY-----
'''

        ret = x509.create_certificate(text=True,
                                      signing_private_key=ca_key,
                                      CN='Redacted Root CA',
                                      O='Redacted',
                                      C='BE',
                                      ST='Antwerp',
                                      L='Local Town',
                                      Email='certadm@example.org',
                                      basicConstraints="critical CA:true",
                                      keyUsage="critical cRLSign, keyCertSign",
                                      subjectKeyIdentifier='hash',
                                      authorityKeyIdentifier='keyid,issuer:always',
                                      days_valid=3650,
                                      days_remaining=0)
        self.assertIn('BEGIN CERTIFICATE', ret)

    @skipIf(not HAS_M2CRYPTO, 'Skipping, M2Crypto is unavailble')
    def test_create_crl(self):
        ca_key = '''
-----BEGIN RSA PRIVATE KEY-----
MIICWwIBAAKBgQCjdjbgL4kQ8Lu73xeRRM1q3C3K3ptfCLpyfw38LRnymxaoJ6ls
pNSx2dU1uJ89YKFlYLo1QcEk4rJ2fdIjarV0kuNCY3rC8jYUp9BpAU5Z6p9HKeT1
2rTPH81JyjbQDR5PyfCyzYOQtpwpB4zIUUK/Go7tTm409xGKbbUFugJNgQIDAQAB
AoGAF24we34U1ZrMLifSRv5nu3OIFNZHyx2DLDpOFOGaII5edwgIXwxZeIzS5Ppr
yO568/8jcdLVDqZ4EkgCwRTgoXRq3a1GLHGFmBdDNvWjSTTMLoozuM0t2zjRmIsH
hUd7tnai9Lf1Bp5HlBEhBU2gZWk+SXqLvxXe74/+BDAj7gECQQDRw1OPsrgTvs3R
3MNwX6W8+iBYMTGjn6f/6rvEzUs/k6rwJluV7n8ISNUIAxoPy5g5vEYK6Ln/Ttc7
u0K1KNlRAkEAx34qcxjuswavL3biNGE+8LpDJnJx1jaNWoH+ObuzYCCVMusdT2gy
kKuq9ytTDgXd2qwZpIDNmscvReFy10glMQJAXebMz3U4Bk7SIHJtYy7OKQzn0dMj
35WnRV81c2Jbnzhhu2PQeAvt/i1sgEuzLQL9QEtSJ6wLJ4mJvImV0TdaIQJAAYyk
TcKK0A8kOy0kMp3yvDHmJZ1L7wr7bBGIZPBlQ0Ddh8i1sJExm1gJ+uN2QKyg/XrK
tDFf52zWnCdVGgDwcQJALW/WcbSEK+JVV6KDJYpwCzWpKIKpBI0F6fdCr1G7Xcwj
c9bcgp7D7xD+TxWWNj4CSXEccJgGr91StV+gFg4ARQ==
-----END RSA PRIVATE KEY-----
'''

        ca_cert = x509.create_certificate(text=True,
                                          signing_private_key=ca_key,
                                          CN='Redacted Root CA',
                                          O='Redacted',
                                          C='BE',
                                          ST='Antwerp',
                                          L='Local Town',
                                          Email='certadm@example.org',
                                          basicConstraints="critical CA:true",
                                          keyUsage="critical cRLSign, keyCertSign",
                                          subjectKeyIdentifier='hash',
                                          authorityKeyIdentifier='keyid,issuer:always',
                                          days_valid=3650,
                                          days_remaining=0)

        with tempfile.NamedTemporaryFile('w+', delete=False) as ca_key_file:
            ca_key_file.write(ca_key)
            ca_key_file.flush()

        with tempfile.NamedTemporaryFile('w+', delete=False) as ca_cert_file:
            ca_cert_file.write(ca_cert)
            ca_cert_file.flush()

        with tempfile.NamedTemporaryFile('w+', delete=False) as ca_crl_file:
            x509.create_crl(path=ca_crl_file.name,
                            text=False,
                            signing_private_key=ca_key_file.name,
                            signing_private_key_passphrase=None,
                            signing_cert=ca_cert_file.name,
                            revoked=None,
                            include_expired=False,
                            days_valid=100,
                            digest='sha512')

        with salt.utils.files.fopen(ca_crl_file.name, 'r') as crl_file:
            crl = crl_file.read()

        os.remove(ca_key_file.name)
        os.remove(ca_cert_file.name)
        os.remove(ca_crl_file.name)

        # Ensure that a CRL was actually created
        self.assertIn('BEGIN X509 CRL', crl)

    @skipIf(not HAS_M2CRYPTO, 'Skipping, M2Crypto is unavailble')
    def test_revoke_certificate_with_crl(self):
        ca_key = '''
-----BEGIN RSA PRIVATE KEY-----
MIICWwIBAAKBgQCjdjbgL4kQ8Lu73xeRRM1q3C3K3ptfCLpyfw38LRnymxaoJ6ls
pNSx2dU1uJ89YKFlYLo1QcEk4rJ2fdIjarV0kuNCY3rC8jYUp9BpAU5Z6p9HKeT1
2rTPH81JyjbQDR5PyfCyzYOQtpwpB4zIUUK/Go7tTm409xGKbbUFugJNgQIDAQAB
AoGAF24we34U1ZrMLifSRv5nu3OIFNZHyx2DLDpOFOGaII5edwgIXwxZeIzS5Ppr
yO568/8jcdLVDqZ4EkgCwRTgoXRq3a1GLHGFmBdDNvWjSTTMLoozuM0t2zjRmIsH
hUd7tnai9Lf1Bp5HlBEhBU2gZWk+SXqLvxXe74/+BDAj7gECQQDRw1OPsrgTvs3R
3MNwX6W8+iBYMTGjn6f/6rvEzUs/k6rwJluV7n8ISNUIAxoPy5g5vEYK6Ln/Ttc7
u0K1KNlRAkEAx34qcxjuswavL3biNGE+8LpDJnJx1jaNWoH+ObuzYCCVMusdT2gy
kKuq9ytTDgXd2qwZpIDNmscvReFy10glMQJAXebMz3U4Bk7SIHJtYy7OKQzn0dMj
35WnRV81c2Jbnzhhu2PQeAvt/i1sgEuzLQL9QEtSJ6wLJ4mJvImV0TdaIQJAAYyk
TcKK0A8kOy0kMp3yvDHmJZ1L7wr7bBGIZPBlQ0Ddh8i1sJExm1gJ+uN2QKyg/XrK
tDFf52zWnCdVGgDwcQJALW/WcbSEK+JVV6KDJYpwCzWpKIKpBI0F6fdCr1G7Xcwj
c9bcgp7D7xD+TxWWNj4CSXEccJgGr91StV+gFg4ARQ==
-----END RSA PRIVATE KEY-----
'''
        # Issue the CA certificate (self-signed)
        ca_cert = x509.create_certificate(text=True,
                                          signing_private_key=ca_key,
                                          CN='Redacted Root CA',
                                          O='Redacted',
                                          C='BE',
                                          ST='Antwerp',
                                          L='Local Town',
                                          Email='certadm@example.org',
                                          basicConstraints="critical CA:true",
                                          keyUsage="critical cRLSign, keyCertSign",
                                          subjectKeyIdentifier='hash',
                                          authorityKeyIdentifier='keyid,issuer:always',
                                          days_valid=3650,
                                          days_remaining=0)

        # Sign a client certificate with the CA
        server_cert = x509.create_certificate(text=True,
                                              signing_private_key=ca_key,
                                              signing_cert=ca_cert,
                                              CN='Redacted Normal Certificate',
                                              O='Redacted',
                                              C='BE',
                                              ST='Antwerp',
                                              L='Local Town',
                                              Email='certadm@example.org',
                                              basicConstraints="critical CA:false",
                                              keyUsage="critical keyEncipherment",
                                              subjectKeyIdentifier='hash',
                                              authorityKeyIdentifier='keyid,issuer:always',
                                              days_valid=365,
                                              days_remaining=0)

        # Save CA cert + key and server cert to disk as PEM files
        with tempfile.NamedTemporaryFile('w+', delete=False) as ca_key_file:
            ca_key_file.write(ca_key)
            ca_key_file.flush()

        with tempfile.NamedTemporaryFile('w+', delete=False) as ca_cert_file:
            ca_cert_file.write(ca_cert)
            ca_cert_file.flush()

        with tempfile.NamedTemporaryFile('w+', delete=False) as server_cert_file:
            server_cert_file.write(server_cert)
            server_cert_file.flush()

        # Revoke server CRL
        revoked = [
            {
                'certificate': server_cert_file.name,
                'revocation_date': '2015-03-01 00:00:00'
            }
        ]
        with tempfile.NamedTemporaryFile('w+', delete=False) as ca_crl_file:
            x509.create_crl(path=ca_crl_file.name,
                            text=False,
                            signing_private_key=ca_key_file.name,
                            signing_private_key_passphrase=None,
                            signing_cert=ca_cert_file.name,
                            revoked=revoked,
                            include_expired=False,
                            days_valid=100,
                            digest='sha512')

        # Retrieve serial number from server certificate
        server_cert_details = x509.read_certificate(server_cert_file.name)
        serial_number = server_cert_details['Serial Number'].replace(':', '')
        serial_number = salt.utils.stringutils.to_str(serial_number)

        # Retrieve CRL as text
        crl = M2Crypto.X509.load_crl(ca_crl_file.name).as_text()

        # Cleanup
        os.remove(ca_key_file.name)
        os.remove(ca_cert_file.name)
        os.remove(ca_crl_file.name)
        os.remove(server_cert_file.name)

        # Ensure that the correct server cert serial is amongst
        # the revoked certificates
        self.assertIn(serial_number, crl)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Rupesh Tare <rupesht@saltstack.com>
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.utils.path
import salt.modules.at as at


@skipIf(NO_MOCK, NO_MOCK_REASON)
class AtTestCase(TestCase, LoaderModuleMockMixin):
    '''
    TestCase for the salt.modules.at module
    '''

    def setup_loader_modules(self):
        return {at: {}}

    atq_output = {'jobs': [{'date': '2014-12-11', 'job': 101, 'queue': 'A',
                            'tag': '', 'time': '19:48:47', 'user': 'B'}]}

    @classmethod
    def tearDownClass(cls):
        del cls.atq_output

    def test_atq_not_available(self):
        '''
        Tests the at.atq not available for any type of os_family.
        '''
        with patch('salt.modules.at._cmd', MagicMock(return_value=None)):
            with patch.dict(at.__grains__, {'os_family': 'RedHat'}):
                self.assertEqual(at.atq(), '\'at.atq\' is not available.')

            with patch.dict(at.__grains__, {'os_family': ''}):
                self.assertEqual(at.atq(), '\'at.atq\' is not available.')

    def test_atq_no_jobs_available(self):
        '''
        Tests the no jobs available for any type of os_family.
        '''
        with patch('salt.modules.at._cmd', MagicMock(return_value='')):
            with patch.dict(at.__grains__, {'os_family': 'RedHat'}):
                self.assertDictEqual(at.atq(), {'jobs': []})

            with patch.dict(at.__grains__, {'os_family': ''}):
                self.assertDictEqual(at.atq(), {'jobs': []})

    def test_atq_list(self):
        '''
        Tests the list all queued and running jobs.
        '''
        with patch('salt.modules.at._cmd') as salt_modules_at__cmd_mock:
            salt_modules_at__cmd_mock.return_value = '101\tThu Dec 11 \
            19:48:47 2014 A B'
            with patch.dict(at.__grains__, {'os_family': '', 'os': ''}):
                self.assertDictEqual(at.atq(), {'jobs': [{'date': '2014-12-11',
                                                          'job': 101,
                                                          'queue': 'A',
                                                          'tag': '',
                                                          'time': '19:48:00',
                                                          'user': 'B'}]})

            salt_modules_at__cmd_mock.return_value = '101\t2014-12-11 \
            19:48:47 A B'
            with patch.dict(at.__grains__, {'os_family': 'RedHat', 'os': ''}):
                self.assertDictEqual(at.atq(), {'jobs': [{'date': '2014-12-11',
                                                          'job': 101,
                                                          'queue': 'A',
                                                          'tag': '',
                                                          'time': '19:48:47',
                                                          'user': 'B'}]})

            salt_modules_at__cmd_mock.return_value = 'SALT: Dec 11, \
            2014 19:48 A 101 B'
            with patch.dict(at.__grains__, {'os_family': '', 'os': 'OpenBSD'}):
                self.assertDictEqual(at.atq(), {'jobs': [{'date': '2014-12-11',
                                                          'job': '101',
                                                          'queue': 'B',
                                                          'tag': '',
                                                          'time': '19:48:00',
                                                          'user': 'A'}]})

    def test_atrm(self):
        """
        Tests for remove jobs from the queue.
        """
        with patch('salt.modules.at.atq', MagicMock(return_value=self.atq_output)):
            with patch.object(salt.utils.path, 'which', return_value=None):
                self.assertEqual(at.atrm(), "'at.atrm' is not available.")

            with patch.object(salt.utils.path, 'which', return_value=True):
                self.assertDictEqual(at.atrm(), {'jobs': {'removed': [],
                                                          'tag': None}})

            with patch.object(at, '_cmd', return_value=True):
                with patch.object(salt.utils.path, 'which', return_value=True):
                    self.assertDictEqual(at.atrm('all'),
                                         {'jobs': {'removed': ['101'],
                                                   'tag': None}})

            with patch.object(at, '_cmd', return_value=True):
                with patch.object(salt.utils.path, 'which', return_value=True):
                    self.assertDictEqual(at.atrm(101),
                                         {'jobs': {'removed': ['101'],
                                                   'tag': None}})

            with patch.object(at, '_cmd', return_value=None):
                self.assertEqual(at.atrm(101), '\'at.atrm\' is not available.')

    def test_jobcheck(self):
        """
        Tests for check the job from queue.
        """
        with patch('salt.modules.at.atq', MagicMock(return_value=self.atq_output)):
            self.assertDictEqual(at.jobcheck(),
                                 {'error': 'You have given a condition'})

            self.assertDictEqual(at.jobcheck(runas='foo'),
                                 {'note': 'No match jobs or time format error',
                                  'jobs': []})

            self.assertDictEqual(at.jobcheck(runas='B', tag='', hour=19, minute=48,
                                             day=11, month=12, Year=2014),
                                 {'jobs': [{'date': '2014-12-11',
                                            'job': 101,
                                            'queue': 'A',
                                            'tag': '',
                                            'time': '19:48:47',
                                            'user': 'B'}]})

    def test_at(self):
        """
        Tests for add a job to the queue.
        """
        with patch('salt.modules.at.atq', MagicMock(return_value=self.atq_output)):
            self.assertDictEqual(at.at(), {'jobs': []})

            with patch.object(salt.utils.path, 'which', return_value=None):
                self.assertEqual(at.at('12:05am', '/sbin/reboot', tag='reboot'),
                                 "'at.at' is not available.")

            with patch.object(salt.utils.path, 'which', return_value=True):
                with patch.dict(at.__grains__, {'os_family': 'RedHat'}):
                    mock = MagicMock(return_value=None)
                    with patch.dict(at.__salt__, {'cmd.run': mock}):
                        self.assertEqual(at.at('12:05am', '/sbin/reboot',
                                               tag='reboot'),
                                         "'at.at' is not available.")

                    mock = MagicMock(return_value='Garbled time')
                    with patch.dict(at.__salt__, {'cmd.run': mock}):
                        self.assertDictEqual(at.at('12:05am', '/sbin/reboot',
                                                   tag='reboot'),
                                             {'jobs': [],
                                              'error': 'invalid timespec'})

                    mock = MagicMock(return_value='warning: commands\nA B')
                    with patch.dict(at.__salt__, {'cmd.run': mock}):
                        with patch.dict(at.__grains__, {'os': 'OpenBSD'}):
                            self.assertDictEqual(at.at('12:05am', '/sbin/reboot',
                                                       tag='reboot'),
                                                 {'jobs': [{'date': '2014-12-11',
                                                            'job': 101,
                                                            'queue': 'A',
                                                            'tag': '',
                                                            'time': '19:48:47',
                                                            'user': 'B'}]})

                with patch.dict(at.__grains__, {'os_family': ''}):
                    mock = MagicMock(return_value=None)
                    with patch.dict(at.__salt__, {'cmd.run': mock}):
                        self.assertEqual(at.at('12:05am', '/sbin/reboot',
                                               tag='reboot'),
                                         "'at.at' is not available.")

    def test_atc(self):
        """
            Tests for atc
        """
        with patch.object(at, '_cmd', return_value=None):
            self.assertEqual(at.atc(101), '\'at.atc\' is not available.')

        with patch.object(at, '_cmd', return_value=''):
            self.assertDictEqual(at.atc(101),
                                 {'error': 'invalid job id \'101\''})

        with patch.object(at, '_cmd',
                          return_value='101\tThu Dec 11 19:48:47 2014 A B'):
            self.assertEqual(at.atc(101), '101\tThu Dec 11 19:48:47 2014 A B')



# -*- coding: utf-8 -*-
'''
Test case for the YAML SDB module
'''

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

# Import Salt libs
import salt.sdb.yaml as sdb


@skipIf(NO_MOCK, NO_MOCK_REASON)
class TestYamlRenderer(TestCase):
    '''
    Test case for the YAML SDB module
    '''

    def test_plaintext(self):
        '''
        Retrieve a value from the top level of the dictionary
        '''
        plain = {'foo': 'bar'}
        with patch('salt.sdb.yaml._get_values', MagicMock(return_value=plain)):
            self.assertEqual(sdb.get('foo'), 'bar')

    def test_nested(self):
        '''
        Retrieve a value from a nested level of the dictionary
        '''
        plain = {'foo': {'bar': 'baz'}}
        with patch('salt.sdb.yaml._get_values', MagicMock(return_value=plain)):
            self.assertEqual(sdb.get('foo:bar'), 'baz')

    def test_encrypted(self):
        '''
        Assume the content is plaintext if GPG is not configured
        '''
        plain = {'foo': 'bar'}
        with patch('salt.sdb.yaml._decrypt', MagicMock(return_value=plain)):
            with patch('salt.sdb.yaml._get_values', MagicMock(return_value=None)):
                self.assertEqual(sdb.get('foo', profile={'gpg': True}), 'bar')



# -*- coding: utf-8 -*-
'''
Tests for the salt-run command
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.case import ShellCase
from tests.support.unit import skipIf

try:
    import libnacl.secret  # pylint: disable=unused-import
    import libnacl.sealed  # pylint: disable=unused-import
    HAS_LIBNACL = True
except ImportError:
    HAS_LIBNACL = False


@skipIf(not HAS_LIBNACL, 'skipping test_nacl, libnacl is unavailable')
class NaclTest(ShellCase):
    '''
    Test the nacl runner
    '''
    def test_keygen(self):
        '''
        Test keygen
        '''
        # Store the data
        ret = self.run_run_plus(
            'nacl.keygen',
        )
        self.assertIn('pk', ret['return'])
        self.assertIn('sk', ret['return'])

    def test_enc(self):
        '''
        Test keygen
        '''
        # Store the data
        ret = self.run_run_plus(
            'nacl.keygen',
        )
        self.assertIn('pk', ret['return'])
        self.assertIn('sk', ret['return'])
        pk = ret['return']['pk']
        sk = ret['return']['sk']

        unencrypted_data = 'hello'

        # Encrypt with pk
        ret = self.run_run_plus(
            'nacl.enc',
            data=unencrypted_data,
            pk=pk,
        )
        self.assertIn('return', ret)

    def test_enc_dec(self):
        '''
        Store, list, fetch, then flush data
        '''
        # Store the data
        ret = self.run_run_plus(
            'nacl.keygen',
        )
        self.assertIn('pk', ret['return'])
        self.assertIn('sk', ret['return'])
        pk = ret['return']['pk']
        sk = ret['return']['sk']

        unencrypted_data = b'hello'

        # Encrypt with pk
        ret = self.run_run_plus(
            'nacl.enc',
            data=unencrypted_data,
            pk=pk,
        )
        self.assertIn('return', ret)
        encrypted_data = ret['return']

        # Decrypt with sk
        ret = self.run_run_plus(
            'nacl.dec',
            data=encrypted_data,
            sk=sk,
        )
        self.assertIn('return', ret)
        self.assertEqual(unencrypted_data, ret['return'])



# -*- coding: utf-8 -*-
'''
Test ext_nodes master_tops module
'''

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import subprocess
import textwrap

# Import Salt Testing libs
from tests.support.unit import TestCase, skipIf
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import patch, MagicMock, NO_MOCK, NO_MOCK_REASON

# Import Salt libs
import salt.utils.stringutils
import salt.tops.ext_nodes as ext_nodes


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ExtNodesTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {
            ext_nodes: {
                '__opts__': {
                    'master_tops': {
                        # Since ext_nodes runs the command with shell=True,
                        # this will keep "command not found" errors from
                        # showing up on the console. We'll be mocking the
                        # communicate results anyway.
                        'ext_nodes': 'echo',
                    }
                }
            }
        }

    def test_ext_nodes(self):
        '''
        Confirm that subprocess.Popen works as expected and does not raise an
        exception (see https://github.com/saltstack/salt/pull/46863).
        '''
        stdout = salt.utils.stringutils.to_bytes(textwrap.dedent('''\
            classes:
              - one
              - two'''))
        communicate_mock = MagicMock(return_value=(stdout, None))
        with patch.object(subprocess.Popen, 'communicate', communicate_mock):
            ret = ext_nodes.top(opts={'id': 'foo'})
        self.assertEqual(ret, {'base': ['one', 'two']})

    def test_ext_nodes_with_environment(self):
        '''
        Same as above, but also tests that the matches are assigned to the proper
        environment if one is returned by the ext_nodes command.
        '''
        stdout = salt.utils.stringutils.to_bytes(textwrap.dedent('''\
            classes:
              - one
              - two
            environment: dev'''))
        communicate_mock = MagicMock(return_value=(stdout, None))
        with patch.object(subprocess.Popen, 'communicate', communicate_mock):
            ret = ext_nodes.top(opts={'id': 'foo'})
        self.assertEqual(ret, {'dev': ['one', 'two']})



# -*- coding: utf-8 -*-
'''
    :codeauthor: Rupesh Tare <rupesht@saltstack.com>
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.http as http
import salt.utils.http


@skipIf(NO_MOCK, NO_MOCK_REASON)
class HttpTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.http
    '''
    def setup_loader_modules(self):
        return {http: {}}

    def test_query(self):
        '''
        Test for Query a resource, and decode the return data
        '''
        with patch.object(salt.utils.http, 'query', return_value='A'):
            self.assertEqual(http.query('url'), 'A')



# -*- coding: utf-8 -*-
'''
Send events covering process status
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals
import logging

# Import third party libs
# pylint: disable=import-error
try:
    import salt.utils.psutil_compat as psutil
    HAS_PSUTIL = True
except ImportError:
    HAS_PSUTIL = False

from salt.ext.six.moves import map

# pylint: enable=import-error

log = logging.getLogger(__name__)  # pylint: disable=invalid-name

__virtualname__ = 'ps'


def __virtual__():
    if not HAS_PSUTIL:
        return (False, 'cannot load ps beacon: psutil not available')
    return __virtualname__


def validate(config):
    '''
    Validate the beacon configuration
    '''
    # Configuration for ps beacon should be a list of dicts
    if not isinstance(config, list):
        return False, ('Configuration for ps beacon must be a list.')
    else:
        _config = {}
        list(map(_config.update, config))

        if 'processes' not in _config:
            return False, ('Configuration for ps beacon requires processes.')
        else:
            if not isinstance(_config['processes'], dict):
                return False, ('Processes for ps beacon must be a dictionary.')

    return True, 'Valid beacon configuration'


def beacon(config):
    '''
    Scan for processes and fire events

    Example Config

    .. code-block:: yaml

        beacons:
          ps:
            - processes:
                salt-master: running
                mysql: stopped

    The config above sets up beacons to check that
    processes are running or stopped.
    '''
    ret = []
    procs = []
    for proc in psutil.process_iter():
        _name = proc.name()
        if _name not in procs:
            procs.append(_name)

    _config = {}
    list(map(_config.update, config))

    for process in _config.get('processes', {}):
        ret_dict = {}
        if _config['processes'][process] == 'running':
            if process in procs:
                ret_dict[process] = 'Running'
                ret.append(ret_dict)
        elif _config['processes'][process] == 'stopped':
            if process not in procs:
                ret_dict[process] = 'Stopped'
                ret.append(ret_dict)
        else:
            if process not in procs:
                ret_dict[process] = False
                ret.append(ret_dict)
    return ret



# -*- coding: utf-8 -*-
'''
    :codeauthor: Nicole Thomas <nicole@saltstack.com>
'''

# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Libs
import salt.modules.mac_sysctl as mac_sysctl
from salt.exceptions import CommandExecutionError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    MagicMock,
    mock_open,
    patch,
    NO_MOCK,
    NO_MOCK_REASON,
    DEFAULT
)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class DarwinSysctlTestCase(TestCase, LoaderModuleMockMixin):
    '''
    TestCase for salt.modules.mac_sysctl module
    '''
    def setup_loader_modules(self):
        return {mac_sysctl: {}}

    def test_get(self):
        '''
        Tests the return of get function
        '''
        mock_cmd = MagicMock(return_value='foo')
        with patch.dict(mac_sysctl.__salt__, {'cmd.run': mock_cmd}):
            self.assertEqual(mac_sysctl.get('kern.ostype'), 'foo')

    def test_assign_cmd_failed(self):
        '''
        Tests if the assignment was successful or not
        '''
        cmd = {'pid': 3548, 'retcode': 1, 'stderr': '',
               'stdout': 'net.inet.icmp.icmplim: 250 -> 50'}
        mock_cmd = MagicMock(return_value=cmd)
        with patch.dict(mac_sysctl.__salt__, {'cmd.run_all': mock_cmd}):
            self.assertRaises(CommandExecutionError,
                              mac_sysctl.assign,
                              'net.inet.icmp.icmplim', 50)

    def test_assign(self):
        '''
        Tests the return of successful assign function
        '''
        cmd = {'pid': 3548, 'retcode': 0, 'stderr': '',
               'stdout': 'net.inet.icmp.icmplim: 250 -> 50'}
        ret = {'net.inet.icmp.icmplim': '50'}
        mock_cmd = MagicMock(return_value=cmd)
        with patch.dict(mac_sysctl.__salt__, {'cmd.run_all': mock_cmd}):
            self.assertEqual(mac_sysctl.assign(
                'net.inet.icmp.icmplim', 50), ret)

    def test_persist_no_conf_failure(self):
        '''
        Tests adding of config file failure
        '''
        read_data = IOError(13, 'Permission denied', '/file')
        with patch('salt.utils.files.fopen', mock_open(read_data=read_data)), \
                patch('os.path.isfile', MagicMock(return_value=False)):
            self.assertRaises(CommandExecutionError,
                              mac_sysctl.persist,
                              'net.inet.icmp.icmplim',
                              50, config=None)

    def test_persist_no_conf_success(self):
        '''
        Tests successful add of config file when it did not already exist
        '''
        config = '/etc/sysctl.conf'
        isfile_mock = MagicMock(
            side_effect=lambda x: False if x == config else DEFAULT
        )
        with patch('salt.utils.files.fopen', mock_open()) as m_open, \
                patch('os.path.isfile', isfile_mock):
            mac_sysctl.persist('net.inet.icmp.icmplim', 50, config=config)
            # We only should have opened the one file
            num_handles = len(m_open.filehandles)
            assert num_handles == 1, num_handles
            writes = m_open.write_calls()
            # We should have called .write() only once, with the expected
            # content
            num_writes = len(writes)
            assert num_writes == 1, num_writes
            assert writes[0] == '#\n# Kernel sysctl configuration\n#\n', writes[0]

    def test_persist_success(self):
        '''
        Tests successful write to existing sysctl file
        '''
        config = '/etc/sysctl.conf'
        to_write = '#\n# Kernel sysctl configuration\n#\n'
        writelines_calls = [[
            '#\n',
            '# Kernel sysctl configuration\n',
            '#\n',
            'net.inet.icmp.icmplim=50\n',
        ]]
        isfile_mock = MagicMock(
            side_effect=lambda x: True if x == config else DEFAULT
        )
        with patch('salt.utils.files.fopen', mock_open(read_data=to_write)) as m_open, \
                patch('os.path.isfile', isfile_mock):
            mac_sysctl.persist('net.inet.icmp.icmplim', 50, config=config)
            # We only should have opened the one file
            num_handles = len(m_open.filehandles)
            assert num_handles == 1, num_handles
            writes = m_open.writelines_calls()
            assert writes == writelines_calls, writes



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.case import SSHCase
from tests.support.unit import skipIf

# Import Salt Libs
import salt.utils.platform


@skipIf(salt.utils.platform.is_windows(), 'salt-ssh not available on Windows')
class SSHPillarTest(SSHCase):
    '''
    testing pillar with salt-ssh
    '''
    def test_pillar_items(self):
        '''
        test pillar.items with salt-ssh
        '''
        ret = self.run_function('pillar.items')
        self.assertDictContainsSubset({'monty': 'python'}, ret)
        self.assertDictContainsSubset(
            {'knights': ['Lancelot', 'Galahad', 'Bedevere', 'Robin']},
            ret)

    def test_pillar_get(self):
        '''
        test pillar.get with salt-ssh
        '''
        ret = self.run_function('pillar.get', ['monty'])
        self.assertEqual(ret, 'python')

    def test_pillar_get_doesnotexist(self):
        '''
        test pillar.get when pillar does not exist with salt-ssh
        '''
        ret = self.run_function('pillar.get', ['doesnotexist'])
        self.assertEqual(ret, '')



# -*- coding: utf-8 -*-
'''
Tests for the Git state
'''

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import functools
import inspect
import os
import shutil
import socket
import string
import tempfile

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.helpers import with_tempdir
from tests.support.mixins import SaltReturnAssertsMixin
from tests.support.paths import TMP

# Import salt libs
import salt.utils.files
import salt.utils.path
from salt.utils.versions import LooseVersion as _LooseVersion
from salt.ext.six.moves.urllib.parse import urlparse  # pylint: disable=no-name-in-module

TEST_REPO = 'https://github.com/saltstack/salt-test-repo.git'


def __check_git_version(caller, min_version, skip_msg):
    '''
    Common logic for version check
    '''
    if inspect.isclass(caller):
        actual_setup = getattr(caller, 'setUp', None)

        def setUp(self, *args, **kwargs):
            if not salt.utils.path.which('git'):
                self.skipTest('git is not installed')
            git_version = self.run_function('git.version')
            if _LooseVersion(git_version) < _LooseVersion(min_version):
                self.skipTest(skip_msg.format(min_version, git_version))
            if actual_setup is not None:
                actual_setup(self, *args, **kwargs)
        caller.setUp = setUp
        return caller

    @functools.wraps(caller)
    def wrapper(self, *args, **kwargs):
        if not salt.utils.path.which('git'):
            self.skipTest('git is not installed')
        git_version = self.run_function('git.version')
        if _LooseVersion(git_version) < _LooseVersion(min_version):
            self.skipTest(skip_msg.format(min_version, git_version))
        return caller(self, *args, **kwargs)
    return wrapper


def ensure_min_git(caller=None, min_version='1.6.5'):
    '''
    Skip test if minimum supported git version is not installed
    '''
    if caller is None:
        return functools.partial(ensure_min_git, min_version=min_version)

    return __check_git_version(
        caller,
        min_version,
        'git {0} or newer required to run this test (detected {1})'
    )


def uses_git_opts(caller):
    '''
    Skip test if git_opts is not supported

    IMPORTANT! This decorator should be at the bottom of any decorators added
    to a given function.
    '''
    min_version = '1.7.2'
    return __check_git_version(
        caller,
        min_version,
        'git_opts only supported in git {0} and newer (detected {1})'
    )


class WithGitMirror(object):
    def __init__(self, repo_url, **kwargs):
        self.repo_url = repo_url
        if 'dir' not in kwargs:
            kwargs['dir'] = TMP
        self.kwargs = kwargs

    def __call__(self, func):
        self.func = func
        return functools.wraps(func)(
            lambda testcase, *args, **kwargs: self.wrap(testcase, *args, **kwargs)  # pylint: disable=W0108
        )

    def wrap(self, testcase, *args, **kwargs):
        # Get temp dir paths
        mirror_dir = tempfile.mkdtemp(**self.kwargs)
        admin_dir = tempfile.mkdtemp(**self.kwargs)
        clone_dir = tempfile.mkdtemp(**self.kwargs)
        # Clean up the directories, we want git to actually create them
        os.rmdir(mirror_dir)
        os.rmdir(admin_dir)
        os.rmdir(clone_dir)
        # Create a URL to clone
        mirror_url = 'file://' + mirror_dir
        # Mirror the repo
        testcase.run_function(
            'git.clone', [mirror_dir], url=TEST_REPO, opts='--mirror')
        # Make sure the directory for the mirror now exists
        assert os.path.exists(mirror_dir)
        # Clone to the admin dir
        ret = testcase.run_state('git.latest', name=mirror_url, target=admin_dir)
        ret = ret[next(iter(ret))]
        assert os.path.exists(admin_dir)

        try:
            # Run the actual function with three arguments added:
            #   1. URL for the test to use to clone
            #   2. Cloned admin dir for making/pushing changes to the mirror
            #   3. Yet-nonexistant clone_dir for the test function to use as a
            #      destination for cloning.
            return self.func(testcase, mirror_url, admin_dir, clone_dir, *args, **kwargs)
        finally:
            shutil.rmtree(mirror_dir, ignore_errors=True)
            shutil.rmtree(admin_dir, ignore_errors=True)
            shutil.rmtree(clone_dir, ignore_errors=True)


with_git_mirror = WithGitMirror


@ensure_min_git
class GitTest(ModuleCase, SaltReturnAssertsMixin):
    '''
    Validate the git state
    '''
    def setUp(self):
        domain = urlparse(TEST_REPO).netloc
        try:
            if hasattr(socket, 'setdefaulttimeout'):
                # 10 second dns timeout
                socket.setdefaulttimeout(10)
            socket.gethostbyname(domain)
        except socket.error:
            msg = 'error resolving {0}, possible network issue?'
            self.skipTest(msg.format(domain))

    def tearDown(self):
        # Reset the dns timeout after the test is over
        socket.setdefaulttimeout(None)

    def _head(self, cwd):
        return self.run_function('git.rev_parse', [cwd, 'HEAD'])

    @with_tempdir(create=False)
    def test_latest(self, target):
        '''
        git.latest
        '''
        ret = self.run_state(
            'git.latest',
            name=TEST_REPO,
            target=target
        )
        self.assertSaltTrueReturn(ret)
        self.assertTrue(os.path.isdir(os.path.join(target, '.git')))

    @with_tempdir(create=False)
    def test_latest_with_rev_and_submodules(self, target):
        '''
        git.latest
        '''
        ret = self.run_state(
            'git.latest',
            name=TEST_REPO,
            rev='develop',
            target=target,
            submodules=True
        )
        self.assertSaltTrueReturn(ret)
        self.assertTrue(os.path.isdir(os.path.join(target, '.git')))

    @with_tempdir(create=False)
    def test_latest_failure(self, target):
        '''
        git.latest
        '''
        ret = self.run_state(
            'git.latest',
            name='https://youSpelledGitHubWrong.com/saltstack/salt-test-repo.git',
            rev='develop',
            target=target,
            submodules=True
        )
        self.assertSaltFalseReturn(ret)
        self.assertFalse(os.path.isdir(os.path.join(target, '.git')))

    @with_tempdir()
    def test_latest_empty_dir(self, target):
        '''
        git.latest
        '''
        ret = self.run_state(
            'git.latest',
            name=TEST_REPO,
            rev='develop',
            target=target,
            submodules=True
        )
        self.assertSaltTrueReturn(ret)
        self.assertTrue(os.path.isdir(os.path.join(target, '.git')))

    @with_tempdir(create=False)
    def test_latest_unless_no_cwd_issue_6800(self, target):
        '''
        cwd=target was being passed to _run_check which blew up if
        target dir did not already exist.
        '''
        ret = self.run_state(
            'git.latest',
            name=TEST_REPO,
            rev='develop',
            target=target,
            unless='test -e {0}'.format(target),
            submodules=True
        )
        self.assertSaltTrueReturn(ret)
        self.assertTrue(os.path.isdir(os.path.join(target, '.git')))

    @with_tempdir(create=False)
    def test_numeric_rev(self, target):
        '''
        git.latest with numeric revision
        '''
        ret = self.run_state(
            'git.latest',
            name=TEST_REPO,
            rev=0.11,
            target=target,
            submodules=True,
            timeout=120
        )
        self.assertSaltTrueReturn(ret)
        self.assertTrue(os.path.isdir(os.path.join(target, '.git')))

    @with_tempdir(create=False)
    def test_latest_with_local_changes(self, target):
        '''
        Ensure that we fail the state when there are local changes and succeed
        when force_reset is True.
        '''
        # Clone repo
        ret = self.run_state(
            'git.latest',
            name=TEST_REPO,
            target=target
        )
        self.assertSaltTrueReturn(ret)
        self.assertTrue(os.path.isdir(os.path.join(target, '.git')))

        # Make change to LICENSE file.
        with salt.utils.files.fopen(os.path.join(target, 'LICENSE'), 'a') as fp_:
            fp_.write('Lorem ipsum dolor blah blah blah....\n')

        # Make sure that we now have uncommitted changes
        self.assertTrue(self.run_function('git.diff', [target, 'HEAD']))

        # Re-run state with force_reset=False
        ret = self.run_state(
            'git.latest',
            name=TEST_REPO,
            target=target,
            force_reset=False
        )
        self.assertSaltTrueReturn(ret)
        self.assertEqual(
            ret[next(iter(ret))]['comment'],
            ('Repository {0} is up-to-date, but with local changes. Set '
             '\'force_reset\' to True to purge local changes.'.format(target))
        )

        # Now run the state with force_reset=True
        ret = self.run_state(
            'git.latest',
            name=TEST_REPO,
            target=target,
            force_reset=True
        )
        self.assertSaltTrueReturn(ret)

        # Make sure that we no longer have uncommitted changes
        self.assertFalse(self.run_function('git.diff', [target, 'HEAD']))

    @with_git_mirror(TEST_REPO)
    @uses_git_opts
    def test_latest_fast_forward(self, mirror_url, admin_dir, clone_dir):
        '''
        Test running git.latest state a second time after changes have been
        made to the remote repo.
        '''
        # Clone the repo
        ret = self.run_state('git.latest', name=mirror_url, target=clone_dir)
        ret = ret[next(iter(ret))]
        assert ret['result']

        # Make a change to the repo by editing the file in the admin copy
        # of the repo and committing.
        head_pre = self._head(admin_dir)
        with salt.utils.files.fopen(os.path.join(admin_dir, 'LICENSE'), 'a') as fp_:
            fp_.write('Hello world!')
        self.run_function(
            'git.commit', [admin_dir, 'added a line'],
            git_opts='-c user.name="Foo Bar" -c user.email=foo@bar.com',
            opts='-a',
        )
        # Make sure HEAD is pointing to a new SHA so we know we properly
        # committed our change.
        head_post = self._head(admin_dir)
        assert head_pre != head_post

        # Push the change to the mirror
        # NOTE: the test will fail if the salt-test-repo's default branch
        # is changed.
        self.run_function('git.push', [admin_dir, 'origin', 'develop'])

        # Re-run the git.latest state on the clone_dir
        ret = self.run_state('git.latest', name=mirror_url, target=clone_dir)
        ret = ret[next(iter(ret))]
        assert ret['result']

        # Make sure that the clone_dir now has the correct SHA
        assert head_post == self._head(clone_dir)

    @with_tempdir(create=False)
    def _changed_local_branch_helper(self, target, rev, hint):
        '''
        We're testing two almost identical cases, the only thing that differs
        is the rev used for the git.latest state.
        '''
        # Clone repo
        ret = self.run_state(
            'git.latest',
            name=TEST_REPO,
            rev=rev,
            target=target
        )
        self.assertSaltTrueReturn(ret)

        # Check out a new branch in the clone and make a commit, to ensure
        # that when we re-run the state, it is not a fast-forward change
        self.run_function('git.checkout', [target, 'new_branch'], opts='-b')
        with salt.utils.files.fopen(os.path.join(target, 'foo'), 'w'):
            pass
        self.run_function('git.add', [target, '.'])
        self.run_function(
            'git.commit', [target, 'add file'],
            git_opts='-c user.name="Foo Bar" -c user.email=foo@bar.com',
        )

        # Re-run the state, this should fail with a specific hint in the
        # comment field.
        ret = self.run_state(
            'git.latest',
            name=TEST_REPO,
            rev=rev,
            target=target
        )
        self.assertSaltFalseReturn(ret)

        comment = ret[next(iter(ret))]['comment']
        self.assertTrue(hint in comment)

    @uses_git_opts
    def test_latest_changed_local_branch_rev_head(self):
        '''
        Test for presence of hint in failure message when the local branch has
        been changed and a the rev is set to HEAD

        This test will fail if the default branch for the salt-test-repo is
        ever changed.
        '''
        self._changed_local_branch_helper(  # pylint: disable=no-value-for-parameter
            'HEAD',
            'The default remote branch (develop) differs from the local '
            'branch (new_branch)'
        )

    @uses_git_opts
    def test_latest_changed_local_branch_rev_develop(self):
        '''
        Test for presence of hint in failure message when the local branch has
        been changed and a non-HEAD rev is specified
        '''
        self._changed_local_branch_helper(  # pylint: disable=no-value-for-parameter
            'develop',
            'The desired rev (develop) differs from the name of the local '
            'branch (new_branch)'
        )

    @uses_git_opts
    @with_tempdir(create=False)
    @with_tempdir()
    def test_latest_updated_remote_rev(self, name, target):
        '''
        Ensure that we don't exit early when checking for a fast-forward
        '''
        # Initialize a new git repository
        self.run_function('git.init', [name])

        # Add and commit a file
        with salt.utils.files.fopen(os.path.join(name, 'foo.txt'), 'w') as fp_:
            fp_.write('Hello world\n')
        self.run_function('git.add', [name, '.'])
        self.run_function(
            'git.commit', [name, 'initial commit'],
            git_opts='-c user.name="Foo Bar" -c user.email=foo@bar.com',
        )

        # Run the state to clone the repo we just created
        ret = self.run_state(
            'git.latest',
            name=name,
            target=target,
        )
        self.assertSaltTrueReturn(ret)

        # Add another commit
        with salt.utils.files.fopen(os.path.join(name, 'foo.txt'), 'w') as fp_:
            fp_.write('Added a line\n')
        self.run_function(
            'git.commit', [name, 'added a line'],
            git_opts='-c user.name="Foo Bar" -c user.email=foo@bar.com',
            opts='-a',
        )

        # Run the state again. It should pass, if it doesn't then there was
        # a problem checking whether or not the change is a fast-forward.
        ret = self.run_state(
            'git.latest',
            name=name,
            target=target,
        )
        self.assertSaltTrueReturn(ret)

    @with_tempdir(create=False)
    def test_latest_depth(self, target):
        '''
        Test running git.latest state using the "depth" argument to limit the
        history. See #45394.
        '''
        ret = self.run_state(
            'git.latest',
            name=TEST_REPO,
            rev='HEAD',
            target=target,
            depth=1
        )
        # HEAD is not a branch, this should fail
        self.assertSaltFalseReturn(ret)
        self.assertIn(
            'must be set to the name of a branch',
            ret[next(iter(ret))]['comment']
        )

        ret = self.run_state(
            'git.latest',
            name=TEST_REPO,
            rev='non-default-branch',
            target=target,
            depth=1
        )
        self.assertSaltTrueReturn(ret)
        self.assertTrue(os.path.isdir(os.path.join(target, '.git')))

    @with_git_mirror(TEST_REPO)
    @uses_git_opts
    def test_latest_sync_tags(self, mirror_url, admin_dir, clone_dir):
        '''
        Test that a removed tag is properly reported as such and removed in the
        local clone, and that new tags are reported as new.
        '''
        tag1 = 'mytag1'
        tag2 = 'mytag2'

        # Add and push a tag
        self.run_function('git.tag', [admin_dir, tag1])
        self.run_function('git.push', [admin_dir, 'origin', tag1])

        # Clone the repo
        ret = self.run_state('git.latest', name=mirror_url, target=clone_dir)
        ret = ret[next(iter(ret))]
        assert ret['result']

        # Now remove the tag
        self.run_function('git.push', [admin_dir, 'origin', ':{0}'.format(tag1)])
        # Add and push another tag
        self.run_function('git.tag', [admin_dir, tag2])
        self.run_function('git.push', [admin_dir, 'origin', tag2])

        # Re-run the state with sync_tags=False. This should NOT delete the tag
        # from the local clone, but should report that a tag has been added.
        ret = self.run_state('git.latest',
                             name=mirror_url,
                             target=clone_dir,
                             sync_tags=False)
        ret = ret[next(iter(ret))]
        assert ret['result']
        # Make ABSOLUTELY SURE both tags are present, since we shouldn't have
        # removed tag1.
        all_tags = self.run_function('git.list_tags', [clone_dir])
        assert tag1 in all_tags
        assert tag2 in all_tags
        # Make sure the reported changes are correct
        expected_changes = {'new_tags': [tag2]}
        assert ret['changes'] == expected_changes, ret['changes']

        # Re-run the state with sync_tags=True. This should remove the local
        # tag, since it doesn't exist in the remote repository.
        ret = self.run_state('git.latest',
                             name=mirror_url,
                             target=clone_dir,
                             sync_tags=True)
        ret = ret[next(iter(ret))]
        assert ret['result']
        # Make ABSOLUTELY SURE the expected tags are present/gone
        all_tags = self.run_function('git.list_tags', [clone_dir])
        assert tag1 not in all_tags
        assert tag2 in all_tags
        # Make sure the reported changes are correct
        expected_changes = {'deleted_tags': [tag1]}
        assert ret['changes'] == expected_changes, ret['changes']

    @with_tempdir(create=False)
    def test_cloned(self, target):
        '''
        Test git.cloned state
        '''
        # Test mode
        ret = self.run_state(
            'git.cloned',
            name=TEST_REPO,
            target=target,
            test=True)
        ret = ret[next(iter(ret))]
        assert ret['result'] is None
        assert ret['changes'] == {
            'new': '{0} => {1}'.format(TEST_REPO, target)
        }
        assert ret['comment'] == '{0} would be cloned to {1}'.format(
            TEST_REPO,
            target
        )

        # Now actually run the state
        ret = self.run_state(
            'git.cloned',
            name=TEST_REPO,
            target=target)
        ret = ret[next(iter(ret))]
        assert ret['result'] is True
        assert ret['changes'] == {
            'new': '{0} => {1}'.format(TEST_REPO, target)
        }
        assert ret['comment'] == '{0} cloned to {1}'.format(TEST_REPO, target)

        # Run the state again to test idempotence
        ret = self.run_state(
            'git.cloned',
            name=TEST_REPO,
            target=target)
        ret = ret[next(iter(ret))]
        assert ret['result'] is True
        assert not ret['changes']
        assert ret['comment'] == 'Repository already exists at {0}'.format(target)

        # Run the state again to test idempotence (test mode)
        ret = self.run_state(
            'git.cloned',
            name=TEST_REPO,
            target=target,
            test=True)
        ret = ret[next(iter(ret))]
        assert not ret['changes']
        assert ret['result'] is True
        assert ret['comment'] == 'Repository already exists at {0}'.format(target)

    @with_tempdir(create=False)
    def test_cloned_with_branch(self, target):
        '''
        Test git.cloned state with branch provided
        '''
        old_branch = 'master'
        new_branch = 'develop'
        bad_branch = 'thisbranchdoesnotexist'

        # Test mode
        ret = self.run_state(
            'git.cloned',
            name=TEST_REPO,
            target=target,
            branch=old_branch,
            test=True)
        ret = ret[next(iter(ret))]
        assert ret['result'] is None
        assert ret['changes'] == {
            'new': '{0} => {1}'.format(TEST_REPO, target)
        }
        assert ret['comment'] == (
            '{0} would be cloned to {1} with branch \'{2}\''.format(
                TEST_REPO,
                target,
                old_branch
            )
        )

        # Now actually run the state
        ret = self.run_state(
            'git.cloned',
            name=TEST_REPO,
            target=target,
            branch=old_branch)
        ret = ret[next(iter(ret))]
        assert ret['result'] is True
        assert ret['changes'] == {
            'new': '{0} => {1}'.format(TEST_REPO, target)
        }
        assert ret['comment'] == (
            '{0} cloned to {1} with branch \'{2}\''.format(
                TEST_REPO,
                target,
                old_branch
            )
        )

        # Run the state again to test idempotence
        ret = self.run_state(
            'git.cloned',
            name=TEST_REPO,
            target=target,
            branch=old_branch)
        ret = ret[next(iter(ret))]
        assert ret['result'] is True
        assert not ret['changes']
        assert ret['comment'] == (
            'Repository already exists at {0} '
            'and is checked out to branch \'{1}\''.format(target, old_branch)
        )

        # Run the state again to test idempotence (test mode)
        ret = self.run_state(
            'git.cloned',
            name=TEST_REPO,
            target=target,
            test=True,
            branch=old_branch)
        ret = ret[next(iter(ret))]
        assert ret['result'] is True
        assert not ret['changes']
        assert ret['comment'] == (
            'Repository already exists at {0} '
            'and is checked out to branch \'{1}\''.format(target, old_branch)
        )

        # Change branch (test mode)
        ret = self.run_state(
            'git.cloned',
            name=TEST_REPO,
            target=target,
            branch=new_branch,
            test=True)
        ret = ret[next(iter(ret))]
        assert ret['result'] is None
        assert ret['changes'] == {
            'branch': {'old': old_branch, 'new': new_branch}
        }
        assert ret['comment'] == 'Branch would be changed to \'{0}\''.format(
            new_branch
        )

        # Now really change the branch
        ret = self.run_state(
            'git.cloned',
            name=TEST_REPO,
            target=target,
            branch=new_branch)
        ret = ret[next(iter(ret))]
        assert ret['result'] is True
        assert ret['changes'] == {
            'branch': {'old': old_branch, 'new': new_branch}
        }
        assert ret['comment'] == 'Branch changed to \'{0}\''.format(
            new_branch
        )

        # Change back to original branch. This tests that we don't attempt to
        # checkout a new branch (i.e. git checkout -b) for a branch that exists
        # locally, as that would fail.
        ret = self.run_state(
            'git.cloned',
            name=TEST_REPO,
            target=target,
            branch=old_branch)
        ret = ret[next(iter(ret))]
        assert ret['result'] is True
        assert ret['changes'] == {
            'branch': {'old': new_branch, 'new': old_branch}
        }
        assert ret['comment'] == 'Branch changed to \'{0}\''.format(
            old_branch
        )

        # Test switching to a nonexistant branch. This should fail.
        ret = self.run_state(
            'git.cloned',
            name=TEST_REPO,
            target=target,
            branch=bad_branch)
        ret = ret[next(iter(ret))]
        assert ret['result'] is False
        assert not ret['changes']
        assert ret['comment'].startswith(
            'Failed to change branch to \'{0}\':'.format(bad_branch)
        )

    @with_tempdir(create=False)
    @ensure_min_git(min_version='1.7.10')
    def test_cloned_with_nonexistant_branch(self, target):
        '''
        Test git.cloned state with a nonexistant branch provided
        '''
        branch = 'thisbranchdoesnotexist'

        # Test mode
        ret = self.run_state(
            'git.cloned',
            name=TEST_REPO,
            target=target,
            branch=branch,
            test=True)
        ret = ret[next(iter(ret))]
        assert ret['result'] is None
        assert ret['changes']
        assert ret['comment'] == (
            '{0} would be cloned to {1} with branch \'{2}\''.format(
                TEST_REPO,
                target,
                branch
            )
        )

        # Now actually run the state
        ret = self.run_state(
            'git.cloned',
            name=TEST_REPO,
            target=target,
            branch=branch)
        ret = ret[next(iter(ret))]
        assert ret['result'] is False
        assert not ret['changes']
        assert ret['comment'].startswith('Clone failed:')
        assert 'not found in upstream origin' in ret['comment']

    @with_tempdir(create=False)
    def test_present(self, name):
        '''
        git.present
        '''
        ret = self.run_state(
            'git.present',
            name=name,
            bare=True
        )
        self.assertSaltTrueReturn(ret)
        self.assertTrue(os.path.isfile(os.path.join(name, 'HEAD')))

    @with_tempdir()
    def test_present_failure(self, name):
        '''
        git.present
        '''
        fname = os.path.join(name, 'stoptheprocess')

        with salt.utils.files.fopen(fname, 'a'):
            pass

        ret = self.run_state(
            'git.present',
            name=name,
            bare=True
        )
        self.assertSaltFalseReturn(ret)
        self.assertFalse(os.path.isfile(os.path.join(name, 'HEAD')))

    @with_tempdir()
    def test_present_empty_dir(self, name):
        '''
        git.present
        '''
        ret = self.run_state(
            'git.present',
            name=name,
            bare=True
        )
        self.assertSaltTrueReturn(ret)
        self.assertTrue(os.path.isfile(os.path.join(name, 'HEAD')))

    @with_tempdir()
    def test_config_set_value_with_space_character(self, name):
        '''
        git.config
        '''
        self.run_function('git.init', [name])

        ret = self.run_state(
            'git.config_set',
            name='user.name',
            value='foo bar',
            repo=name,
            **{'global': False})
        self.assertSaltTrueReturn(ret)


@ensure_min_git
@uses_git_opts
class LocalRepoGitTest(ModuleCase, SaltReturnAssertsMixin):
    '''
    Tests which do no require connectivity to github.com
    '''
    @with_tempdir()
    @with_tempdir()
    @with_tempdir()
    def test_renamed_default_branch(self, repo, admin, target):
        '''
        Test the case where the remote branch has been removed
        https://github.com/saltstack/salt/issues/36242
        '''
        # Create bare repo
        self.run_function('git.init', [repo], bare=True)
        # Clone bare repo
        self.run_function('git.clone', [admin], url=repo)
        # Create, add, commit, and push file
        with salt.utils.files.fopen(os.path.join(admin, 'foo'), 'w'):
            pass
        self.run_function('git.add', [admin, '.'])
        self.run_function(
            'git.commit', [admin, 'initial commit'],
            git_opts='-c user.name="Foo Bar" -c user.email=foo@bar.com',
        )
        self.run_function('git.push', [admin], remote='origin', ref='master')

        # Rename remote 'master' branch to 'develop'
        os.rename(
            os.path.join(repo, 'refs', 'heads', 'master'),
            os.path.join(repo, 'refs', 'heads', 'develop')
        )

        # Run git.latest state. This should successfully clone and fail with a
        # specific error in the comment field.
        ret = self.run_state(
            'git.latest',
            name=repo,
            target=target,
            rev='develop',
        )
        self.assertSaltFalseReturn(ret)
        self.assertEqual(
            ret[next(iter(ret))]['comment'],
            'Remote HEAD refers to a ref that does not exist. '
            'This can happen when the default branch on the '
            'remote repository is renamed or deleted. If you '
            'are unable to fix the remote repository, you can '
            'work around this by setting the \'branch\' argument '
            '(which will ensure that the named branch is created '
            'if it does not already exist).\n\n'
            'Changes already made: {0} cloned to {1}'
            .format(repo, target)
        )
        self.assertEqual(
            ret[next(iter(ret))]['changes'],
            {'new': '{0} => {1}'.format(repo, target)}
        )

        # Run git.latest state again. This should fail again, with a different
        # error in the comment field, and should not change anything.
        ret = self.run_state(
            'git.latest',
            name=repo,
            target=target,
            rev='develop',
        )
        self.assertSaltFalseReturn(ret)
        self.assertEqual(
            ret[next(iter(ret))]['comment'],
            'Cannot set/unset upstream tracking branch, local '
            'HEAD refers to nonexistent branch. This may have '
            'been caused by cloning a remote repository for which '
            'the default branch was renamed or deleted. If you '
            'are unable to fix the remote repository, you can '
            'work around this by setting the \'branch\' argument '
            '(which will ensure that the named branch is created '
            'if it does not already exist).'
        )
        self.assertEqual(ret[next(iter(ret))]['changes'], {})

        # Run git.latest state again with a branch manually set. This should
        # checkout a new branch and the state should pass.
        ret = self.run_state(
            'git.latest',
            name=repo,
            target=target,
            rev='develop',
            branch='develop',
        )
        # State should succeed
        self.assertSaltTrueReturn(ret)
        self.assertSaltCommentRegexpMatches(
            ret,
            'New branch \'develop\' was checked out, with origin/develop '
            r'\([0-9a-f]{7}\) as a starting point'
        )
        # Only the revision should be in the changes dict.
        self.assertEqual(
            list(ret[next(iter(ret))]['changes'].keys()),
            ['revision']
        )
        # Since the remote repo was incorrectly set up, the local head should
        # not exist (therefore the old revision should be None).
        self.assertEqual(
            ret[next(iter(ret))]['changes']['revision']['old'],
            None
        )
        # Make sure the new revision is a SHA (40 chars, all hex)
        self.assertTrue(
            len(ret[next(iter(ret))]['changes']['revision']['new']) == 40)
        self.assertTrue(
            all([x in string.hexdigits for x in
                 ret[next(iter(ret))]['changes']['revision']['new']])
        )



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, unicode_literals

# Import Salt Testing libs
from tests.support.unit import TestCase

# Import Salt libs
from salt.utils.listdiffer import list_diff

from salt.utils import dictdiffer
NONE = dictdiffer.RecursiveDictDiffer.NONE_VALUE


class ListDictDifferTestCase(TestCase):

    def setUp(self):
        old_list = [{'key': 1, 'value': 'foo1', 'int_value': 101},
                    {'key': 2, 'value': 'foo2', 'int_value': 102},
                    {'key': 3, 'value': 'foo3', 'int_value': 103}]
        new_list = [{'key': 1, 'value': 'foo1', 'int_value': 101},
                    {'key': 2, 'value': 'foo2', 'int_value': 112},
                    {'key': 5, 'value': 'foo5', 'int_value': 105}]
        self.list_diff = list_diff(old_list, new_list, key='key')

    def tearDown(self):
        for attrname in ('list_diff',):
            try:
                delattr(self, attrname)
            except AttributeError:
                continue

    def test_added(self):
        self.assertEqual(len(self.list_diff.added), 1)
        self.assertDictEqual(self.list_diff.added[0],
                             {'key': 5, 'value': 'foo5', 'int_value': 105})

    def test_removed(self):
        self.assertEqual(len(self.list_diff.removed), 1)
        self.assertDictEqual(self.list_diff.removed[0],
                             {'key': 3, 'value': 'foo3', 'int_value': 103})

    def test_diffs(self):
        self.assertEqual(len(self.list_diff.diffs), 3)
        self.assertDictEqual(self.list_diff.diffs[0],
                             {2: {'int_value': {'new': 112, 'old': 102}}})
        self.assertDictEqual(self.list_diff.diffs[1],
                             # Added items
                             {5: {'int_value': {'new': 105, 'old': NONE},
                                  'key': {'new': 5, 'old': NONE},
                                  'value': {'new': 'foo5', 'old': NONE}}})
        self.assertDictEqual(self.list_diff.diffs[2],
                             # Removed items
                             {3: {'int_value': {'new': NONE, 'old': 103},
                                  'key': {'new': NONE, 'old': 3},
                                  'value': {'new': NONE, 'old': 'foo3'}}})

    def test_new_values(self):
        self.assertEqual(len(self.list_diff.new_values), 2)
        self.assertDictEqual(self.list_diff.new_values[0],
                             {'key': 2, 'int_value': 112})
        self.assertDictEqual(self.list_diff.new_values[1],
                             {'key': 5, 'value': 'foo5', 'int_value': 105})

    def test_old_values(self):
        self.assertEqual(len(self.list_diff.old_values), 2)
        self.assertDictEqual(self.list_diff.old_values[0],
                             {'key': 2, 'int_value': 102})
        self.assertDictEqual(self.list_diff.old_values[1],
                             {'key': 3, 'value': 'foo3', 'int_value': 103})

    def test_changed_all(self):
        self.assertEqual(self.list_diff.changed(selection='all'),
                         ['key.2.int_value', 'key.5.int_value', 'key.5.value',
                          'key.3.int_value', 'key.3.value'])

    def test_changed_intersect(self):
        self.assertEqual(self.list_diff.changed(selection='intersect'),
                         ['key.2.int_value'])

    def test_changes_str(self):
        self.assertEqual(self.list_diff.changes_str,
                         '\tidentified by key 2:\n'
                         '\tint_value from 102 to 112\n'
                         '\tidentified by key 3:\n'
                         '\twill be removed\n'
                         '\tidentified by key 5:\n'
                         '\twill be added\n')



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.unit import skipIf
from tests.support.helpers import destructiveTest, flaky

# Import Salt Libs
import salt.utils.platform


@flaky
@skipIf(not salt.utils.platform.is_windows(), 'Tests for only Windows')
class NTPTest(ModuleCase):
    '''
    Validate windows ntp module
    '''
    @destructiveTest
    def test_ntp_set_servers(self):
        '''
        test ntp get and set servers
        '''
        ntp_srv = 'pool.ntp.org'
        set_srv = self.run_function('ntp.set_servers', [ntp_srv])
        self.assertTrue(set_srv)

        get_srv = self.run_function('ntp.get_servers')
        self.assertEqual(ntp_srv, get_srv[0])



# -*- coding: utf-8 -*-
'''
tests for host state
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.case import ModuleCase


class HandleOrderTest(ModuleCase):
    '''
    Validate that ordering works correctly
    '''
    def test_handle_iorder(self):
        '''
        Test the error with multiple states of the same type
        '''
        ret = self.run_function('state.show_low_sls', mods='issue-7649-handle-iorder')

        sorted_chunks = [chunk['name'] for chunk in sorted(ret, key=lambda c: c.get('order'))]

        expected = ['./configure', 'make', 'make install']
        self.assertEqual(expected, sorted_chunks)



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, unicode_literals, print_function
import os

# Import Salt Testing libs
from tests.support.paths import FILES
from tests.support.unit import skipIf
from tests.support.helpers import requires_network

# Import Salt libs
import salt.utils.path
from tests.unit.modules.test_zcbuildout import Base, KNOWN_VIRTUALENV_BINARY_NAMES
import salt.modules.zcbuildout as modbuildout
import salt.states.zcbuildout as buildout
import salt.modules.cmdmod as cmd

ROOT = os.path.join(FILES, 'file/base/buildout')


@skipIf(salt.utils.path.which_bin(KNOWN_VIRTUALENV_BINARY_NAMES) is None,
        "The 'virtualenv' packaged needs to be installed")
class BuildoutTestCase(Base):

    def setup_loader_modules(self):
        module_globals = {
            '__env__': 'base',
            '__opts__': {'test': False},
            '__salt__': {
                'cmd.run_all': cmd.run_all,
                'cmd.run': cmd.run,
                'cmd.retcode': cmd.retcode,
                'buildout.buildout': modbuildout.buildout,
            }
        }
        return {buildout: module_globals, modbuildout: module_globals}

    @requires_network()
    def test_quiet(self):
        c_dir = os.path.join(self.tdir, 'c')
        cret = buildout.installed(c_dir, python=self.py_st, quiet=True)
        self.assertTrue(cret['result'])
        self.assertFalse('OUTPUT:' in cret['comment'])
        self.assertFalse('Log summary:' in cret['comment'])

    @requires_network()
    def test_error(self):
        b_dir = os.path.join(self.tdir, 'e')
        ret = buildout.installed(b_dir, python=self.py_st)
        self.assertTrue(
            'We did not get any expectable '
            'answer from buildout'
            in ret['comment'])
        self.assertTrue(
            'An internal error occurred due to a bug in'
            ' either zc.buildout '
            in ret['comment'])
        self.assertFalse(ret['result'])

    @requires_network()
    def test_installed(self):
        b_dir = os.path.join(self.tdir, 'b')
        ret = buildout.installed(b_dir,
                                 python=self.py_st,
                                 onlyif='/bin/false')
        self.assertEqual(ret['comment'], '\nonlyif condition is false')
        self.assertEqual(ret['result'], True)
        self.assertTrue('/b' in ret['name'])
        b_dir = os.path.join(self.tdir, 'b')
        ret = buildout.installed(b_dir,
                                 python=self.py_st,
                                 unless='/bin/true')
        self.assertEqual(ret['comment'], '\nunless condition is true')
        self.assertEqual(ret['result'], True)
        self.assertTrue('/b' in ret['name'])
        ret = buildout.installed(b_dir, python=self.py_st)
        self.assertEqual(ret['result'], True)
        self.assertTrue('OUTPUT:' in ret['comment'])
        self.assertTrue('Log summary:' in ret['comment'])



# -*- coding: utf-8 -*-
'''
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)

    tests.conftest
    ~~~~~~~~~~~~~~

    Prepare py.test for our test suite
'''

# Import python libs
from __future__ import absolute_import
import os
import sys
import stat
import socket
import logging
from collections import namedtuple

TESTS_DIR = os.path.dirname(
    os.path.normpath(os.path.abspath(__file__))
)
CODE_DIR = os.path.dirname(TESTS_DIR)
os.chdir(CODE_DIR)
try:
    # If we have a system-wide salt module imported, unload it
    import salt
    for module in list(sys.modules):
        if module.startswith(('salt',)):
            try:
                if not sys.modules[module].__file__.startswith(CODE_DIR):
                    sys.modules.pop(module)
            except AttributeError:
                continue
    sys.path.insert(0, CODE_DIR)
except ImportError:
    sys.path.insert(0, CODE_DIR)

# Import test libs
import tests.support.paths  # pylint: disable=unused-import
from tests.integration import TestDaemon

# Import pytest libs
import pytest
from _pytest.terminal import TerminalReporter

# Import 3rd-party libs
import psutil
from salt.ext import six

# Import salt libs
import salt.utils.files
import salt.utils.path
import salt.log.setup
from salt.utils.odict import OrderedDict

# Define the pytest plugins we rely on
pytest_plugins = ['tempdir', 'helpers_namespace']  # pylint: disable=invalid-name

# Define where not to collect tests from
collect_ignore = ['setup.py']

log = logging.getLogger('salt.testsuite')

# Reset logging root handlers
for handler in logging.root.handlers:
    logging.root.removeHandler(handler)


def pytest_tempdir_basename():
    '''
    Return the temporary directory basename for the salt test suite.
    '''
    return 'salt-tests-tmp'


# ----- CLI Options Setup ------------------------------------------------------------------------------------------->
def pytest_addoption(parser):
    '''
    register argparse-style options and ini-style config values.
    '''
    parser.addoption(
        '--sysinfo',
        default=False,
        action='store_true',
        help='Print some system information.'
    )
    parser.addoption(
        '--transport',
        default='zeromq',
        choices=('zeromq', 'raet', 'tcp'),
        help=('Select which transport to run the integration tests with, '
              'zeromq, raet, or tcp. Default: %default')
    )
    test_selection_group = parser.getgroup('Tests Selection')
    test_selection_group.addoption(
        '--ssh',
        '--ssh-tests',
        dest='ssh',
        action='store_true',
        default=False,
        help='Run salt-ssh tests. These tests will spin up a temporary '
             'SSH server on your machine. In certain environments, this '
             'may be insecure! Default: False'
    )
    test_selection_group.addoption(
        '--proxy',
        '--proxy-tests',
        dest='proxy',
        action='store_true',
        default=False,
        help='Run proxy tests'
    )
    test_selection_group.addoption(
        '--run-destructive',
        action='store_true',
        default=False,
        help='Run destructive tests. These tests can include adding '
             'or removing users from your system for example. '
             'Default: False'
    )
    test_selection_group.addoption(
        '--run-expensive',
        action='store_true',
        default=False,
        help='Run expensive tests. These tests usually involve costs '
             'like for example bootstrapping a cloud VM. '
             'Default: False'
    )
    output_options_group = parser.getgroup('Output Options')
    output_options_group.addoption(
        '--output-columns',
        default=80,
        type=int,
        help='Number of maximum columns to use on the output'
    )
    output_options_group.addoption(
        '--no-colors',
        '--no-colours',
        default=False,
        action='store_true',
        help='Disable colour printing.'
    )
    output_options_group.addoption(
        '--sys-stats',
        default=False,
        action='store_true',
        help='Print System CPU and MEM statistics after each test execution.'
    )
# <---- CLI Options Setup --------------------------------------------------------------------------------------------


# ----- CLI Terminal Reporter --------------------------------------------------------------------------------------->
class SaltTerminalReporter(TerminalReporter):
    def __init__(self, config):
        TerminalReporter.__init__(self, config)

    @pytest.hookimpl(trylast=True)
    def pytest_sessionstart(self, session):
        TerminalReporter.pytest_sessionstart(self, session)
        self._session = session

    def pytest_runtest_logreport(self, report):
        TerminalReporter.pytest_runtest_logreport(self, report)
        if self.verbosity <= 0:
            return
        if report.when != 'call':
            return
        if self.config.getoption('--sys-stats') is False:
            return

        test_daemon = getattr(self._session, 'test_daemon', None)
        if self.verbosity == 1:
            line = ' [CPU:{0}%|MEM:{1}%]'.format(psutil.cpu_percent(),
                                               psutil.virtual_memory().percent)
            self._tw.write(line)
            return
        else:
            self.ensure_newline()
            template = ' {}  -  CPU: {:6.2f} %   MEM: {:6.2f} %   SWAP: {:6.2f} %\n'
            self._tw.write(
                template.format(
                    '            System',
                    psutil.cpu_percent(),
                    psutil.virtual_memory().percent,
                    psutil.swap_memory().percent
                )
            )
            for name, psproc in self._session.stats_processes.items():
                with psproc.oneshot():
                    cpu = psproc.cpu_percent()
                    mem = psproc.memory_percent('vms')
                    swap = psproc.memory_percent('swap')
                    self._tw.write(template.format(name, cpu, mem, swap))


def pytest_sessionstart(session):
    session.stats_processes = OrderedDict((
        #('Log Server', test_daemon.log_server),
        ('    Test Suite Run', psutil.Process(os.getpid())),
    ))
# <---- CLI Terminal Reporter ----------------------------------------------------------------------------------------


# ----- Register Markers -------------------------------------------------------------------------------------------->
@pytest.mark.trylast
def pytest_configure(config):
    '''
    called after command line options have been parsed
    and all plugins and initial conftest files been loaded.
    '''
    config.addinivalue_line('norecursedirs', os.path.join(CODE_DIR, 'templates'))
    config.addinivalue_line(
        'markers',
        'destructive_test: Run destructive tests. These tests can include adding '
        'or removing users from your system for example.'
    )
    config.addinivalue_line(
        'markers',
        'skip_if_not_root: Skip if the current user is not `root`.'
    )
    config.addinivalue_line(
        'markers',
        'skip_if_binaries_missing(*binaries, check_all=False, message=None): Skip if '
        'any of the passed binaries are not found in path. If \'check_all\' is '
        '\'True\', then all binaries must be found.'
    )
    config.addinivalue_line(
        'markers',
        'requires_network(only_local_network=False): Skip if no networking is set up. '
        'If \'only_local_network\' is \'True\', only the local network is checked.'
    )

    # Register our terminal reporter
    if not getattr(config, 'slaveinput', None):
        standard_reporter = config.pluginmanager.getplugin('terminalreporter')
        salt_reporter = SaltTerminalReporter(standard_reporter.config)

        config.pluginmanager.unregister(standard_reporter)
        config.pluginmanager.register(salt_reporter, 'terminalreporter')

    # Transplant configuration
    TestDaemon.transplant_configs(transport=config.getoption('--transport'))
# <---- Register Markers ---------------------------------------------------------------------------------------------


# ----- Test Setup -------------------------------------------------------------------------------------------------->
@pytest.hookimpl(tryfirst=True)
def pytest_runtest_setup(item):
    '''
    Fixtures injection based on markers or test skips based on CLI arguments
    '''
    destructive_tests_marker = item.get_marker('destructive_test')
    if destructive_tests_marker is not None:
        if item.config.getoption('--run-destructive') is False:
            pytest.skip('Destructive tests are disabled')
    os.environ['DESTRUCTIVE_TESTS'] = six.text_type(item.config.getoption('--run-destructive'))

    expensive_tests_marker = item.get_marker('expensive_test')
    if expensive_tests_marker is not None:
        if item.config.getoption('--run-expensive') is False:
            pytest.skip('Expensive tests are disabled')
    os.environ['EXPENSIVE_TESTS'] = six.text_type(item.config.getoption('--run-expensive'))

    skip_if_not_root_marker = item.get_marker('skip_if_not_root')
    if skip_if_not_root_marker is not None:
        if os.getuid() != 0:
            pytest.skip('You must be logged in as root to run this test')

    skip_if_binaries_missing_marker = item.get_marker('skip_if_binaries_missing')
    if skip_if_binaries_missing_marker is not None:
        binaries = skip_if_binaries_missing_marker.args
        if len(binaries) == 1:
            if isinstance(binaries[0], (list, tuple, set, frozenset)):
                binaries = binaries[0]
        check_all = skip_if_binaries_missing_marker.kwargs.get('check_all', False)
        message = skip_if_binaries_missing_marker.kwargs.get('message', None)
        if check_all:
            for binary in binaries:
                if salt.utils.path.which(binary) is None:
                    pytest.skip(
                        '{0}The "{1}" binary was not found'.format(
                            message and '{0}. '.format(message) or '',
                            binary
                        )
                    )
        elif salt.utils.path.which_bin(binaries) is None:
            pytest.skip(
                '{0}None of the following binaries was found: {1}'.format(
                    message and '{0}. '.format(message) or '',
                    ', '.join(binaries)
                )
            )

    requires_network_marker = item.get_marker('requires_network')
    if requires_network_marker is not None:
        only_local_network = requires_network_marker.kwargs.get('only_local_network', False)
        has_local_network = False
        # First lets try if we have a local network. Inspired in verify_socket
        try:
            pubsock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            retsock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            pubsock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            pubsock.bind(('', 18000))
            pubsock.close()
            retsock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            retsock.bind(('', 18001))
            retsock.close()
            has_local_network = True
        except socket.error:
            # I wonder if we just have IPV6 support?
            try:
                pubsock = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
                retsock = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
                pubsock.setsockopt(
                    socket.SOL_SOCKET, socket.SO_REUSEADDR, 1
                )
                pubsock.bind(('', 18000))
                pubsock.close()
                retsock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                retsock.bind(('', 18001))
                retsock.close()
                has_local_network = True
            except socket.error:
                # Let's continue
                pass

        if only_local_network is True:
            if has_local_network is False:
                # Since we're only supposed to check local network, and no
                # local network was detected, skip the test
                pytest.skip('No local network was detected')

        # We are using the google.com DNS records as numerical IPs to avoid
        # DNS lookups which could greatly slow down this check
        for addr in ('173.194.41.198', '173.194.41.199', '173.194.41.200',
                     '173.194.41.201', '173.194.41.206', '173.194.41.192',
                     '173.194.41.193', '173.194.41.194', '173.194.41.195',
                     '173.194.41.196', '173.194.41.197'):
            try:
                sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                sock.settimeout(0.25)
                sock.connect((addr, 80))
                sock.close()
                # We connected? Stop the loop
                break
            except socket.error:
                # Let's check the next IP
                continue
            else:
                pytest.skip('No internet network connection was detected')
# <---- Test Setup ---------------------------------------------------------------------------------------------------


# ----- Automatic Markers Setup ------------------------------------------------------------------------------------->
def pytest_collection_modifyitems(items):
    '''
    Automatically add markers to tests based on directory layout
    '''
    for item in items:
        fspath = str(item.fspath)
        if '/integration/' in fspath:
            if 'test_daemon' not in item.fixturenames:
                item.fixturenames.append('test_daemon')
            item.add_marker(pytest.mark.integration)
            for kind in ('cli', 'client', 'cloud', 'fileserver', 'loader', 'minion', 'modules',
                         'netapi', 'output', 'reactor', 'renderers', 'runners', 'sdb', 'shell',
                         'ssh', 'states', 'utils', 'wheel'):
                if '/{0}/'.format(kind) in fspath:
                    item.add_marker(getattr(pytest.mark, kind))
                    break
        if '/unit/' in fspath:
            item.add_marker(pytest.mark.unit)
            for kind in ('acl', 'beacons', 'cli', 'cloud', 'config', 'grains', 'modules', 'netapi',
                         'output', 'pillar', 'renderers', 'runners', 'serializers', 'states',
                         'templates', 'transport', 'utils'):
                if '/{0}/'.format(kind) in fspath:
                    item.add_marker(getattr(pytest.mark, kind))
                    break
# <---- Automatic Markers Setup --------------------------------------------------------------------------------------


# ----- Pytest Helpers ---------------------------------------------------------------------------------------------->
if six.PY2:
    # backport mock_open from the python 3 unittest.mock library so that we can
    # mock read, readline, readlines, and file iteration properly

    file_spec = None

    def _iterate_read_data(read_data):
        # Helper for mock_open:
        # Retrieve lines from read_data via a generator so that separate calls to
        # readline, read, and readlines are properly interleaved
        data_as_list = ['{0}\n'.format(l) for l in read_data.split('\n')]

        if data_as_list[-1] == '\n':
            # If the last line ended in a newline, the list comprehension will have an
            # extra entry that's just a newline.  Remove this.
            data_as_list = data_as_list[:-1]
        else:
            # If there wasn't an extra newline by itself, then the file being
            # emulated doesn't have a newline to end the last line  remove the
            # newline that our naive format() added
            data_as_list[-1] = data_as_list[-1][:-1]

        for line in data_as_list:
            yield line

    @pytest.helpers.mock.register
    def mock_open(mock=None, read_data=''):
        """
        A helper function to create a mock to replace the use of `open`. It works
        for `open` called directly or used as a context manager.

        The `mock` argument is the mock object to configure. If `None` (the
        default) then a `MagicMock` will be created for you, with the API limited
        to methods or attributes available on standard file handles.

        `read_data` is a string for the `read` methoddline`, and `readlines` of the
        file handle to return.  This is an empty string by default.
        """
        _mock = pytest.importorskip('mock', minversion='2.0.0')

        def _readlines_side_effect(*args, **kwargs):
            if handle.readlines.return_value is not None:
                return handle.readlines.return_value
            return list(_data)

        def _read_side_effect(*args, **kwargs):
            if handle.read.return_value is not None:
                return handle.read.return_value
            return ''.join(_data)

        def _readline_side_effect():
            if handle.readline.return_value is not None:
                while True:
                    yield handle.readline.return_value
            for line in _data:
                yield line

        global file_spec
        if file_spec is None:
            file_spec = file  # pylint: disable=undefined-variable

        if mock is None:
            mock = _mock.MagicMock(name='open', spec=open)

        handle = _mock.MagicMock(spec=file_spec)
        handle.__enter__.return_value = handle

        _data = _iterate_read_data(read_data)

        handle.write.return_value = None
        handle.read.return_value = None
        handle.readline.return_value = None
        handle.readlines.return_value = None

        handle.read.side_effect = _read_side_effect
        handle.readline.side_effect = _readline_side_effect()
        handle.readlines.side_effect = _readlines_side_effect

        mock.return_value = handle
        return mock
else:
    @pytest.helpers.mock.register
    def mock_open(mock=None, read_data=''):
        _mock = pytest.importorskip('mock', minversion='2.0.0')
        return _mock.mock_open(mock=mock, read_data=read_data)
# <---- Pytest Helpers -----------------------------------------------------------------------------------------------


# ----- Fixtures Overrides ------------------------------------------------------------------------------------------>
# ----- Generate CLI Scripts ---------------------------------------------------------------------------------------->
@pytest.fixture(scope='session')
def cli_master_script_name():
    '''
    Return the CLI script basename
    '''
    return 'cli_salt_master'


@pytest.fixture(scope='session')
def cli_minion_script_name():
    '''
    Return the CLI script basename
    '''
    return 'cli_salt_minion'


@pytest.fixture(scope='session')
def cli_salt_script_name():
    '''
    Return the CLI script basename
    '''
    return 'cli_salt'


@pytest.fixture(scope='session')
def cli_run_script_name():
    '''
    Return the CLI script basename
    '''
    return 'cli_salt_run'


@pytest.fixture(scope='session')
def cli_key_script_name():
    '''
    Return the CLI script basename
    '''
    return 'cli_salt_key'


@pytest.fixture(scope='session')
def cli_call_script_name():
    '''
    Return the CLI script basename
    '''
    return 'cli_salt_call'


@pytest.fixture(scope='session')
def cli_syndic_script_name():
    '''
    Return the CLI script basename
    '''
    return 'cli_salt_syndic'


@pytest.fixture(scope='session')
def cli_ssh_script_name():
    '''
    Return the CLI script basename
    '''
    return 'cli_salt_ssh'


@pytest.fixture(scope='session')
def cli_bin_dir(tempdir,
                request,
                python_executable_path,
                cli_master_script_name,
                cli_minion_script_name,
                cli_salt_script_name,
                cli_call_script_name,
                cli_key_script_name,
                cli_run_script_name,
                cli_ssh_script_name):
    '''
    Return the path to the CLI script directory to use
    '''
    tmp_cli_scripts_dir = tempdir.join('cli-scrips-bin')
    tmp_cli_scripts_dir.ensure(dir=True)
    cli_bin_dir_path = tmp_cli_scripts_dir.strpath

    # Now that we have the CLI directory created, lets generate the required CLI scripts to run salt's test suite
    script_templates = {
        'salt': [
            'from salt.scripts import salt_main\n',
            'if __name__ == \'__main__\':\n'
            '    salt_main()'
        ],
        'salt-api': [
            'import salt.cli\n',
            'def main():\n',
            '    sapi = salt.cli.SaltAPI()',
            '    sapi.run()\n',
            'if __name__ == \'__main__\':',
            '    main()'
        ],
        'common': [
            'from salt.scripts import salt_{0}\n',
            'if __name__ == \'__main__\':\n',
            '    salt_{0}()'
        ]
    }

    for script_name in (cli_master_script_name,
                        cli_minion_script_name,
                        cli_call_script_name,
                        cli_key_script_name,
                        cli_run_script_name,
                        cli_salt_script_name,
                        cli_ssh_script_name):
        original_script_name = script_name.split('cli_')[-1].replace('_', '-')
        script_path = os.path.join(cli_bin_dir_path, script_name)

        if not os.path.isfile(script_path):
            log.info('Generating {0}'.format(script_path))

            with salt.utils.files.fopen(script_path, 'w') as sfh:
                script_template = script_templates.get(original_script_name, None)
                if script_template is None:
                    script_template = script_templates.get('common', None)
                if script_template is None:
                    raise RuntimeError(
                        'Salt\'s test suite does not know how to handle the "{0}" script'.format(
                            original_script_name
                        )
                    )
                sfh.write(
                    '#!{0}\n\n'.format(python_executable_path) +
                    'import sys\n' +
                    'CODE_DIR="{0}"\n'.format(request.config.startdir.realpath().strpath) +
                    'if CODE_DIR not in sys.path:\n' +
                    '    sys.path.insert(0, CODE_DIR)\n\n' +
                    '\n'.join(script_template).format(original_script_name.replace('salt-', ''))
                )
            fst = os.stat(script_path)
            os.chmod(script_path, fst.st_mode | stat.S_IEXEC)

    # Return the CLI bin dir value
    return cli_bin_dir_path
# <---- Generate CLI Scripts -----------------------------------------------------------------------------------------


# ----- Salt Configuration ------------------------------------------------------------------------------------------>
@pytest.fixture(scope='session')
def session_integration_files_dir(request):
    '''
    Fixture which returns the salt integration files directory path.
    Creates the directory if it does not yet exist.
    '''
    return request.config.startdir.join('tests').join('integration').join('files')


@pytest.fixture(scope='session')
def session_state_tree_root_dir(session_integration_files_dir):
    '''
    Fixture which returns the salt state tree root directory path.
    Creates the directory if it does not yet exist.
    '''
    return session_integration_files_dir.join('file')


@pytest.fixture(scope='session')
def session_pillar_tree_root_dir(session_integration_files_dir):
    '''
    Fixture which returns the salt pillar tree root directory path.
    Creates the directory if it does not yet exist.
    '''
    return session_integration_files_dir.join('pillar')
# <---- Salt Configuration -------------------------------------------------------------------------------------------


# <---- Fixtures Overrides -------------------------------------------------------------------------------------------
# ----- Custom Fixtures Definitions --------------------------------------------------------------------------------->
@pytest.fixture(scope='session')
def test_daemon(request):
    values = (('transport', request.config.getoption('--transport')),
              ('sysinfo', request.config.getoption('--sysinfo')),
              ('no_colors', request.config.getoption('--no-colors')),
              ('output_columns', request.config.getoption('--output-columns')),
              ('ssh', request.config.getoption('--ssh')),
              ('proxy', request.config.getoption('--proxy')))
    options = namedtuple('options', [n for n, v in values])(*[v for n, v in values])
    fake_parser = namedtuple('parser', 'options')(options)

    test_daemon = TestDaemon(fake_parser)
    with test_daemon as test_daemon_running:
        request.session.test_daemon = test_daemon_running
        request.session.stats_processes.update(OrderedDict((
            ('       Salt Master', psutil.Process(test_daemon.master_process.pid)),
            ('       Salt Minion', psutil.Process(test_daemon.minion_process.pid)),
            ('   Salt Sub Minion', psutil.Process(test_daemon.sub_minion_process.pid)),
            ('Salt Syndic Master', psutil.Process(test_daemon.smaster_process.pid)),
            ('       Salt Syndic', psutil.Process(test_daemon.syndic_process.pid)),
        )).items())
        yield
    TestDaemon.clean()
# <---- Custom Fixtures Definitions ----------------------------------------------------------------------------------



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase

# Import Salt libs
import salt.state
from salt.config import minion_config
from salt.template import compile_template_str
import salt.serializers.yamlex as yamlex

basic_template = '''#!yamlex
foo: bar
'''

complex_template = '''#!yamlex
placeholder: {foo: !aggregate {foo: 42}}
placeholder: {foo: !aggregate {bar: null}}
placeholder: {foo: !aggregate {baz: inga}}
'''

SKIP_MESSAGE = '%s is unavailable, do prerequisites have been met?'


class RendererMixin(object):
    def render(self, template, opts=None):
        _config = minion_config(None)
        _config['file_client'] = 'local'
        if opts:
            _config.update(opts)
        _state = salt.state.State(_config)
        return compile_template_str(template,
                                    _state.rend,
                                    _state.opts['renderer'],
                                    _state.opts['renderer_blacklist'],
                                    _state.opts['renderer_whitelist'])


class RendererTests(TestCase, RendererMixin, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        return {yamlex: {}}

    @skipIf(not yamlex.available, SKIP_MESSAGE % 'yamlex')
    def test_basic(self):
        sls_obj = self.render(basic_template)
        assert sls_obj == {'foo': 'bar'}, sls_obj

    @skipIf(not yamlex.available, SKIP_MESSAGE % 'yamlex')
    def test_complex(self):

        sls_obj = self.render(complex_template)
        assert sls_obj == {
            'placeholder': {
                'foo': {
                    'foo': 42,
                    'bar': None,
                    'baz': 'inga'
                }
            }
        }, sls_obj



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch,
    mock_open)

# Import Salt Libs
import salt.states.apache as apache
import salt.utils.files


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ApacheTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.states.apache
    '''
    def setup_loader_modules(self):
        return {apache: {}}

    # 'configfile' function tests: 1

    def test_configfile(self):
        '''
        Test to allows for inputting a yaml dictionary into a file
        for apache configuration files.
        '''
        with patch('os.path.exists', MagicMock(return_value=True)):
            name = '/etc/distro/specific/apache.conf'
            config = 'VirtualHost: this: "*:80"'
            new_config = 'LiteralHost: that: "*:79"'

            ret = {'name': name,
                   'result': True,
                   'changes': {},
                   'comment': ''}

            with patch.object(salt.utils.files, 'fopen', mock_open(read_data=config)):
                mock_config = MagicMock(return_value=config)
                with patch.dict(apache.__salt__, {'apache.config': mock_config}):
                    ret.update({'comment': 'Configuration is up to date.'})
                    self.assertDictEqual(apache.configfile(name, config), ret)

            with patch.object(salt.utils.files, 'fopen', mock_open(read_data=config)):
                mock_config = MagicMock(return_value=new_config)
                with patch.dict(apache.__salt__, {'apache.config': mock_config}):
                    ret.update({'comment': 'Configuration will update.',
                                'changes': {'new': new_config,
                                            'old': config},
                                'result': None})
                    with patch.dict(apache.__opts__, {'test': True}):
                        self.assertDictEqual(apache.configfile(name, new_config), ret)

            with patch.object(salt.utils.files, 'fopen', mock_open(read_data=config)):
                mock_config = MagicMock(return_value=new_config)
                with patch.dict(apache.__salt__, {'apache.config': mock_config}):
                    ret.update({'comment': 'Successfully created configuration.',
                                'result': True})
                    with patch.dict(apache.__opts__, {'test': False}):
                        self.assertDictEqual(apache.configfile(name, config), ret)



# -*- coding: utf-8 -*-
'''
    tests.unit.returners.cassandra_cql_test
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'''

# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function
import ssl

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import patch, MagicMock

# Import salt libs
import salt.modules.cassandra_cql as cassandra_cql
from salt.exceptions import CommandExecutionError

try:
    import cassandra  # pylint: disable=unused-import,wrong-import-position
    HAS_CASSANDRA = True
except ImportError:
    HAS_CASSANDRA = False


@skipIf(
    not HAS_CASSANDRA,
    'Please install the cassandra datastax driver to run cassandra_cql module unit tests.'
)
class CassandraCQLReturnerTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cassandra CQL module
    '''
    def setup_loader_modules(self):
        return {cassandra_cql: {}}

    def test_returns_opts_if_specified(self):
        '''
        If ssl options are present then check that they are parsed and returned
        '''
        options = MagicMock(return_value={
            'cluster': [
                '192.168.50.10', '192.168.50.11', '192.168.50.12'],
            'port': 9000,
            'ssl_options': {
                'ca_certs': '/etc/ssl/certs/ca-bundle.trust.crt',
                'ssl_version': 'PROTOCOL_TLSv1'},
            'username': 'cas_admin'}
        )

        with patch.dict(cassandra_cql.__salt__, {'config.option': options}):

            self.assertEqual(cassandra_cql._get_ssl_opts(), {  # pylint: disable=protected-access
                'ca_certs': '/etc/ssl/certs/ca-bundle.trust.crt', 'ssl_version': ssl.PROTOCOL_TLSv1})  # pylint: disable=no-member

    def test_invalid_protocol_version(self):
        '''
        Check that the protocol version is imported only if it isvalid
        '''
        options = MagicMock(return_value={
            'cluster': [
                '192.168.50.10', '192.168.50.11', '192.168.50.12'],
            'port': 9000,
            'ssl_options': {
                'ca_certs': '/etc/ssl/certs/ca-bundle.trust.crt',
                'ssl_version': 'Invalid'},
            'username': 'cas_admin'}
        )

        with patch.dict(cassandra_cql.__salt__, {'config.option': options}):
            with self.assertRaises(CommandExecutionError):
                cassandra_cql._get_ssl_opts()  # pylint: disable=protected-access

    def test_unspecified_opts(self):
        '''
        Check that it returns None when ssl opts aren't specified
        '''
        with patch.dict(cassandra_cql.__salt__, {'config.option': MagicMock(return_value={})}):
            self.assertEqual(cassandra_cql._get_ssl_opts(),  # pylint: disable=protected-access
                             None)



# -*- coding: utf-8 -*-
'''
Tests for various minion timeouts
'''

# Import Python libs
from __future__ import absolute_import
import os
import sys

import salt.utils.platform

# Import Salt Testing libs
from tests.support.case import ShellCase


class MinionTimeoutTestCase(ShellCase):
    '''
    Test minion timing functions
    '''
    def test_long_running_job(self):
        '''
        Test that we will wait longer than the job timeout for a minion to
        return.
        '''
        # Launch the command
        sleep_length = 30
        if salt.utils.platform.is_windows():
            popen_kwargs = {'env': dict(os.environ, PYTHONPATH=';'.join(sys.path))}
        else:
            popen_kwargs = None
        ret = self.run_salt(
            'minion test.sleep {0}'.format(sleep_length),
            timeout=45,
            catch_stderr=True,
            popen_kwargs=popen_kwargs,
        )
        self.assertTrue(isinstance(ret[0], list), 'Return is not a list. Minion'
                ' may have returned error: {0}'.format(ret))
        self.assertEqual(len(ret[0]), 2, 'Standard out wrong length {}'.format(ret))
        self.assertTrue('True' in ret[0][1], 'Minion did not return True after '
                '{0} seconds. ret={1}'.format(sleep_length, ret))



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
from collections import OrderedDict

# Import Salt Testing libs
from tests.support.unit import TestCase

# Import Salt Libs
import salt.pillar.pepa as pepa


class PepaPillarTestCase(TestCase):
    def test_repeated_keys(self):
        expected_result = {
            "foo": {
                "bar": {
                    "foo": True,
                    "baz": True,
                },
            },
        }
        data = OrderedDict([
            ('foo..bar..foo', True),
            ('foo..bar..baz', True),
        ])
        result = pepa.key_value_to_tree(data)
        self.assertDictEqual(result, expected_result)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Anthony Shaw <anthonyshaw@apache.org>`
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON
)

import tests.support.napalm as napalm_test_support
import salt.modules.napalm_yang_mod as napalm_yang_mod  # NOQA
import salt.modules.napalm_network as napalm_network  # NOQA


TEST_DIFF = {
    'diff1': 'value'
}


class MockNapalmYangModel(object):
    def Root(self):
        return MagicMock()


class MockNapalmYangModels(object):
    openconfig_interfaces = MockNapalmYangModel()


class MockUtils(object):
    def diff(self, *args):
        return TEST_DIFF


class MockNapalmYangModule(object):
    base = MockNapalmYangModel()
    models = MockNapalmYangModels()
    utils = MockUtils()


TEST_CONFIG = {
    'comment': 'Configuration discarded.',
    'already_configured': False,
    'result': True,
    'diff': '[edit interfaces xe-0/0/5]+   description "Adding a description";'
}


def mock_net_load_config(**kwargs):
    return TEST_CONFIG


@skipIf(NO_MOCK, NO_MOCK_REASON)
class NapalmYangModModuleTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        module_globals = {
            '__salt__': {
                'config.option': MagicMock(return_value={
                    'test': {
                        'driver': 'test',
                        'key': '2orgk34kgk34g'
                    }
                }),
                'file.file_exists': napalm_test_support.true,
                'file.join': napalm_test_support.join,
                'file.get_managed': napalm_test_support.get_managed_file,
                'random.hash': napalm_test_support.random_hash,
                'net.load_template': napalm_network.load_template,
                'net.load_config': mock_net_load_config
            }
        }
        module_globals['napalm_yang'] = MockNapalmYangModule()

        return {napalm_yang_mod: module_globals, napalm_network: module_globals}

    def test_diff(self):
        ret = napalm_yang_mod.diff({}, {'test': True}, 'models.openconfig_interfaces')
        assert ret == TEST_DIFF

    def test_diff_list(self):
        '''
        Test it with an actual list
        '''
        ret = napalm_yang_mod.diff({}, {'test': True}, ['models.openconfig_interfaces'])
        assert ret == TEST_DIFF

    def test_parse(self):
        ret = napalm_yang_mod.parse('models.openconfig_interfaces')
        assert ret is not None

    def test_get_config(self):
        ret = napalm_yang_mod.get_config({}, 'models.openconfig_interfaces')
        assert ret is not None

    def test_load_config(self):
        ret = napalm_yang_mod.load_config({}, 'models.openconfig_interfaces')
        assert ret is TEST_CONFIG

    def test_compliance_report(self):
        ret = napalm_yang_mod.compliance_report({}, 'models.openconfig_interfaces')
        assert ret is not None



# -*- coding: utf-8 -*-
'''
Integration tests for the zookeeper states
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.unit import skipIf
from tests.support.case import ModuleCase
from tests.support.helpers import destructiveTest
from tests.support.mixins import SaltReturnAssertsMixin

# Import Salt Libs
import salt.utils.path

try:
    import kazoo  # pylint: disable=import-error,unused-import
    HAS_KAZOO = True
except ImportError:
    HAS_KAZOO = False


@destructiveTest
@skipIf(not salt.utils.path.which('dockerd'), 'Docker not installed')
@skipIf(not HAS_KAZOO, 'kazoo python library not installed')
class ZookeeperTestCase(ModuleCase, SaltReturnAssertsMixin):
    '''
    Test zookeeper states
    '''
    def setUp(self):
        '''
        '''
        self.run_state('docker_image.present', name='zookeeper')
        self.run_state('docker_container.running', name='zookeeper', image='zookeeper', port_bindings='2181:2181')

    def tearDown(self):
        self.run_state('docker_container.stopped', name='zookeeper')
        self.run_state('docker_container.absent', name='zookeeper')
        self.run_state('docker_image.absent', name='docker.io/zookeeper', force=True)

    def test_zookeeper_present(self):
        ret = self.run_state(
            'zookeeper.present',
            name='/test/name',
            value='testuser',
            makepath=True,
        )
        self.assertSaltTrueReturn(ret)

        ret = self.run_state(
            'zookeeper.present',
            name='/test/name',
            value='daniel',
            acls=[
                {'username': 'daniel', 'password': 'test', 'read': True, 'admin': True, 'write': True, },
                {'username': 'testuser', 'password': 'test', 'read': True},
            ],
            profile='prod',
        )
        self.assertSaltTrueReturn(ret)

    def test_zookeeper_absent(self):
        self.run_state(
            'zookeeper.present',
            name='/test/name',
            value='testuser',
            makepath=True,
        )
        ret = self.run_state(
            'zookeeper.absent',
            name='/test/name',
        )
        self.assertSaltTrueReturn(ret)
        self.assertTrue(bool(ret['zookeeper_|-/test/name_|-/test/name_|-absent']['changes']))
        ret = self.run_state(
            'zookeeper.absent',
            name='/test/name',
        )
        self.assertFalse(bool(ret['zookeeper_|-/test/name_|-/test/name_|-absent']['changes']))

    def test_zookeeper_acls(self):
        ret = self.run_state(
            'zookeeper.acls',
            name='/test/name',
            acls=[
                {'username': 'daniel', 'password': 'test', 'read': True, 'admin': True, 'write': True, },
                {'username': 'testuser', 'password': 'test', 'read': True},
            ]
        )
        self.assertSaltFalseReturn(ret)

        ret = self.run_state(
            'zookeeper.present',
            name='/test/name',
            value='testuser',
            makepath=True,
        )

        ret = self.run_state(
            'zookeeper.acls',
            name='/test/name',
            acls=[
                {'username': 'daniel', 'password': 'test', 'read': True, 'admin': True, 'write': True, },
                {'username': 'testuser', 'password': 'test', 'read': True},
            ]
        )
        self.assertSaltTrueReturn(ret)



# -*- coding: utf-8 -*-
'''
Tests for the spm info utility
'''
# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import shutil

# Import Salt Testing libs
from tests.support.case import SPMCase
from tests.support.helpers import destructiveTest


@destructiveTest
class SPMInfoTest(SPMCase):
    '''
    Validate the spm info command
    '''
    def setUp(self):
        self.config = self._spm_config()
        self._spm_build_files(self.config)

    def test_spm_info(self):
        '''
        test spm build
        '''
        self._spm_create_update_repo(self.config)
        install = self.run_spm('install', self.config, 'apache')
        get_info = self.run_spm('info', self.config, 'apache')

        check_info = ['Supported OSes', 'Supported OS', 'installing Apache']
        for info in check_info:
            self.assertIn(info, ''.join(get_info))

    def tearDown(self):
        shutil.rmtree(self._tmp_spm)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.case import SSHCase
from tests.support.unit import skipIf

# Import Salt Libs
import salt.utils.platform


@skipIf(salt.utils.platform.is_windows(), 'salt-ssh not available on Windows')
class SSHGrainsTest(SSHCase):
    '''
    testing grains with salt-ssh
    '''
    def test_grains_items(self):
        '''
        test grains.items with salt-ssh
        '''
        ret = self.run_function('grains.items')
        grain = 'Linux'
        if salt.utils.platform.is_darwin():
            grain = 'Darwin'
        self.assertEqual(ret['kernel'], grain)
        self.assertTrue(isinstance(ret, dict))
